"""
Name: EI1050
Desc: A few simple classes to handle communication with the EI1050 probe
"""

import sys
import threading
import time

import u3
import u6


class EI1050:
    """
    EI1050 class to simplify communication with an EI1050 probe
    """

    U3 = 3
    U6 = 6
    UE9 = 9
    U3_STRING = "U3"
    U6_STRING = "U6"
    UE9_STRING = "UE9"
    U3_DEFAULT_DATA_PIN_NUM = 4
    U3_DEFAULT_CLOCK_PIN_NUM = 5
    U3_DEFAULT_ENABLE_PIN_NUM = 7
    U6_DEFAULT_DATA_PIN_NUM = 0
    U6_DEFAULT_CLOCK_PIN_NUM = 1
    U6_DEFAULT_ENABLE_PIN_NUM = 3
    UE9_DEFAULT_DATA_PIN_NUM = 0
    UE9_DEFAULT_CLOCK_PIN_NUM = 1
    UE9_DEFAULT_ENABLE_PIN_NUM = 3
    FIO_PIN_STATE = 0

    def __init__(self, device, autoUpdate=True, enablePinNum=-1, dataPinNum=-1, clockPinNum=-1, shtOptions=0xc0):
        self.device = device
        self.autoUpdate = autoUpdate
        self.dataPinNum = dataPinNum
        self.clockPinNum = clockPinNum
        self.shtOptions = shtOptions
        self.enablePinNum = enablePinNum
        self.curState = {'StatusReg': None, 'StatusCRC': None,
                         'Temperature': None, 'TemperatureCRC': None,
                         'Humidity': None, 'HumidityCRC': None}

        # Determine device type
        if self.device.__class__.__name__ == EI1050.U3_STRING:
            self.deviceType = EI1050.U3
        elif self.device.__class__.__name__ == EI1050.U6_STRING:
            self.deviceType = EI1050.U6
        elif self.device.__class__.__name__ == EI1050.UE9_STRING:
            self.deviceType = EI1050.UE9
        else:
            raise TypeError('Invalid device passed. Can not get default values.')

        # If not specified otherwise, use class default for the data pin number
        if self.enablePinNum == -1:
            if self.deviceType == EI1050.U3:
                self.enablePinNum = EI1050.U3_DEFAULT_ENABLE_PIN_NUM
            elif self.deviceType == EI1050.U6:
                self.enablePinNum = EI1050.U6_DEFAULT_ENABLE_PIN_NUM
            elif self.deviceType == EI1050.UE9:
                self.enablePinNum = EI1050.UE9_DEFAULT_ENABLE_PIN_NUM
            else:
                raise TypeError('Invalid device passed. Can not get default values.')

        # If not specified otherwise, use class default for the data pin number
        if self.dataPinNum == -1:
            if self.deviceType == EI1050.U3:
                self.dataPinNum = EI1050.U3_DEFAULT_DATA_PIN_NUM
            elif self.deviceType == EI1050.U6:
                self.dataPinNum = EI1050.U6_DEFAULT_DATA_PIN_NUM
            elif self.deviceType == EI1050.UE9:
                self.dataPinNum = EI1050.UE9_DEFAULT_DATA_PIN_NUM
            else:
                raise TypeError('Invalid device passed. Can not get default values.')

        # If not specified otherwise, use class default for the clock pin number
        if self.clockPinNum == -1:
            if self.deviceType == EI1050.U3:
                self.clockPinNum = EI1050.U3_DEFAULT_CLOCK_PIN_NUM
            elif self.deviceType == EI1050.U6:
                self.clockPinNum = EI1050.U6_DEFAULT_CLOCK_PIN_NUM
            elif self.deviceType == EI1050.UE9:
                self.clockPinNum = EI1050.UE9_DEFAULT_CLOCK_PIN_NUM
            else:
                raise TypeError('Invalid device passed. Can not get default values.')

        # Set U3 pins
        if self.deviceType == EI1050.U3:
            self.device.configIO(FIOAnalog=EI1050.FIO_PIN_STATE)

        # Set to write out
        if self.deviceType == EI1050.U3:
            self.device.getFeedback(u3.BitDirWrite(self.enablePinNum, 1))
        elif self.deviceType == EI1050.U6:
            self.device.getFeedback(u6.BitDirWrite(self.enablePinNum, 1))

    def enableAutoUpdate(self):
        """
        Name: EI1050.enableAutoUpdate()
        Desc: Turns on automatic updating of readings
        """
        self.autoUpdate = True

    def disableAutoUpdate(self):
        """
        Name: EI1050.disableAutoUpdate()
        Desc: Truns off automatic updating of readings
        """
        self.autoUpdate = False

    def getStatus(self):
        """
        Name: EI1050.getStatus()
        Desc: A read of the SHT1x status register
        """
        if self.autoUpdate:
            self.update()
        return self.curState['StatusReg']

    def getStatusCRC(self):
        """
        Name: EI1050.getStatusCRC()
        Desc: Get cyclic reduancy check for status
        """
        if self.autoUpdate:
            self.update()
        return self.curState['StatusCRC']

    def getTemperature(self):
        """
        Name: EI1050.getTemperature()
        Desc: Get a temperature reading from the EI1050 probe
        """
        if self.autoUpdate:
            self.update()
        return self.curState['Temperature']

    def getTemperatureCRC(self):
        """
        Name: EI1050.getStatusCRC()
        Desc: Get cyclic redundancy check for temperature reading
        """
        if self.autoUpdate:
            self.update()
        return self.curState['TemperatureCRC']

    def getHumidity(self):
        """
        Name: EI1050.getHumidity()
        Desc: Get a humidity reading from the EI1050 probe
        """
        if self.autoUpdate:
            self.update()
        return self.curState['Humidity']

    def getHumidityCRC(self):
        """
        Name: EI1050.getHumidityCRC()
        Desc: Get cyclic redundancy check for humidity reading
        """
        if self.autoUpdate:
            self.update()
        return self.curState['HumidityCRC']

    def getReading(self):
        """
        Name: EI1050.getReading()
        Desc: Get a reading and create a Reading object with the information
        """
        if self.autoUpdate:
            self.update()
        if 'StatusCRC' in self.curState:
            return Reading(self.curState['StatusReg'], self.curState['StatusCRC'], self.curState['Temperature'], self.curState['TemperatureCRC'], self.curState['Humidity'], self.curState['HumidityCRC'])
        else:
            return Reading(self.curState['StatusReg'], 0, self.curState['Temperature'], self.curState['TemperatureCRC'], self.curState['Humidity'], self.curState['HumidityCRC'])

    def update(self):
        """
        Name: EI1050.update()
        Desc: Gets a fresh set of readings from this probe
        """
        self.writeBitState(self.enablePinNum, 1)  # Enable the probe
        self.curState = self.device.sht1x(self.dataPinNum, self.clockPinNum, self.shtOptions)
        self.writeBitState(self.enablePinNum, 0)  # Disable the probe

    def writeBitState(self, pinNum, state):
        """
        Name: EI1050.writeBitState(pinNum, state)
        Desc: Device independent way to write bit state
        """

        if self.deviceType == EI1050.U3:
            self.device.getFeedback(u3.BitStateWrite(pinNum, state))
        elif self.deviceType == EI1050.U6:
            self.device.getFeedback(u6.BitStateWrite(pinNum, state))
        elif self.deviceType == EI1050.UE9:
            self.device.singleIO(1, pinNum, Dir=1, State=state)


class Reading:
    """
    A class that represents a reading taken from an EI1050 probe
    """

    def __init__(self, status, statusCRC, temperature, temperatureCRC, humidity, humidityCRC):
        self.__status = status
        self.__statusCRC = statusCRC
        self.__temperature = temperature
        self.__temperatureCRC = temperatureCRC
        self.__humidity = humidity
        self.__humidityCRC = humidityCRC

    def getStatus(self):
        """
        Name: Reading.getStatus()
        Desc: Returns that status of the EI1050 probe at the time of this reading
        """
        return self.__status

    def getStatusCRC(self):
        """
        Name: Reading.getStatusCRC()
        Desc: Get the cyclic reduancy check for status at the time of this reading
        """
        return self.__statusCRC

    def getTemperature(self):
        """
        Name: Reading.getTemperature()
        Desc: Get the temperature reported by the EI1050 probe at the time of this reading
        """
        return self.__temperature

    def getTemperatureCRC(self):
        """
        Name: Reading.getStatusCRC()
        Desc: Get cyclic redundancy check for the temperature reading
        """
        return self.__temperatureCRC

    def getHumidity(self):
        """
        Name: Reading.getHumidity()
        Desc: Get the humidity reported by the EI1050 probe at the time of this reading
        """
        return self.__humidity

    def getHumidityCRC(self):
        """
        Name: Reading.getHumidityCRC()
        Desc: Get cyclic redundancy check for humidity reading
        """
        return self.__humidityCRC


class EI1050Reader(threading.Thread):
    """
    A simple threading class to read EI1050 values
    """
    def __init__(self, device, targetQueue, readingDelay=1, autoUpdate=True, enablePinNum=-1, dataPinNum=-1, clockPinNum=-1, shtOptions=0xc0):
        try:
            self.readingDelay = readingDelay  # How long to wait between reads (in sec)
            self.targetQueue = targetQueue  # The queue in which readings will be placed
            self.probe = EI1050(device, autoUpdate, enablePinNum, dataPinNum, clockPinNum, shtOptions)
            self.running = False
            self.exception = None
            threading.Thread.__init__(self, group=None)
        except:
            self.exception = sys.exc_info()

    def stop(self):
        """
        Stops this thread's activity. Note: this may not be immediate
        """
        self.running = False

    def run(self):
        self.running = True
        while self.running:
            try:
                self.targetQueue.put(self.probe.getReading())
                time.sleep(self.readingDelay)
            except:
                self.exception = sys.exc_info()
                self.stop()
                break



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
gdev_i2c.py - I2C support; limited to dongle ELV with Sensors BME280 and TSL2591

"""

###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################


__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"

from   gsup_utils       import *

import gdev_i2c_Dngl_ELV            as ELV      # I2C dongle
import gdev_i2c_Sensor_BME280       as BME280   # I2C sensor
import gdev_i2c_Sensor_TSL2591      as TSL2591  # I2C sensor

try:
    import serial                       # serial port (module has name: 'pyserial'!)
    import serial.tools.list_ports      # allows listing of serial ports
except Exception as e:
    msg  = "\nPython module 'serial' was not found\n"
    msg += "In order to run any I2C device this module is required\n"
    msg += "Verify that 'pyserial' is installed using GeigerLog tool: 'gtools/GLpipcheck.py'"
    exceptPrint(e, msg)
    edprint("Halting GeigerLog")
    playWav("err")
    sys.exit()


ELVdongle           = "ELVdongle"         # ELV USB-I2C Dongle

# I2C Sensors and Modules
bme280              = {
                       "name"   : "BME280",
                       "active" : False,                # TESTING
                       "feat"   : "Temperature, Pressure, Humidity",
                       "addr"   : 0x77,      # (d119)  addr: 0x76, 0x77
                       "type"   : 0x60,      # (d96)   BME280 has chip_ID 0x60
                       "hndl"   : None,      # handle
                       "dngl"   : None,      # connected with dongle
                      }

tsl2591             = {
                       "name"   : "TSL2591",
                       "active" : False,                # TESTING
                       "feat"   : "Light (Vis, IR)",
                       "addr"   : 0x29,      # (d41)  addr: 0x29
                       "type"   : 0x50,      # Device ID 0x50
                       "hndl"   : None,      # handle
                       "dngl"   : None,      # connected with dongle
                      }


def initI2C():
    """Init the ELV dongle with the BME280 sensor"""

    gglobs.I2CConnection     = False
    gglobs.I2CDeviceName     = "I2C Sensors"
    gglobs.I2CDeviceDetected = gglobs.I2CDeviceName
    gglobs.Devices["I2C"][0] = gglobs.I2CDeviceDetected

    # init the ELV dongle
    gglobs.elv            = ELV.ELVdongle()
    success, msg          = gglobs.elv.ELVinit()
    if not success:
        msg2 = "Failure initializing dongle {}: {}".format(gglobs.elv.name, msg)
        edprint(msg2, debug=True)
        return msg2

    tmplts               = "Failure initializing sensor {} on dongle {} with message:<br>{}"

    # init sensor BME280 after linking with ELV dongle
    if bme280["active"]:
        bme280["dngl"] = ELVdongle
        bme280['hndl'] = BME280.SensorBME280(bme280)
        response       = bme280['hndl'].BME280Init()
        if response > "":
            return tmplts.format(bme280["name"], bme280["dngl"], response)

    # init sensor TSL2591 after linking with ELV dongle
    if tsl2591["active"]:
        tsl2591["dngl"] = ELVdongle
        tsl2591['hndl'] = TSL2591.SensorTSL2591(tsl2591)
        response               = tsl2591['hndl'].TSL2591Init()
        if response > "":
            return tmplts.format(bme280["name"], bme280["dngl"], response)

    gglobs.I2CConnection = True

    # setup the queues
    gglobs.QueueBME280  = queue.Queue()
    gglobs.QueueTSL2591 = queue.Queue()

    # setup the thread and start it
    gglobs.I2CThread = I2CReader(gglobs.QueueBME280, gglobs.QueueTSL2591)
    gglobs.I2CThread.start()

    # determine the variables (X is for vis light)
    if gglobs.I2CVariables == "auto": gglobs.I2CVariables    = "T, P, H, X"

    setLoggableVariables("I2C", gglobs.I2CVariables)

    # Save the info to avoid serial port call in conflict with thread!
    gglobs.I2CInfo         = getI2CInfo(extended=False, first=True)
    gglobs.I2CInfoExtended = getI2CInfo(extended=True,  first=True)

    return ""


def terminateI2C():
    """shutting down thread, closing ELV, resetting connection flag"""

    if gglobs.I2CThread != None:
        gglobs.I2CThread.stop() # fist stop threads, then close port!
        # wait for thread to end, but wait not longer than 5 sec
        start = time.time()
        while gglobs.I2CThread.is_alive() and (time.time() - start) < 5:
            time.sleep(0.1)
            pass            # wait for thread to end
        wprint("terminateI2C: thread-status: is alive: ", gglobs.I2CThread.is_alive())

    if gglobs.elv       != None: gglobs.elv.ELVclose()

    gglobs.I2CConnection = False

    return ""


def getI2CValues(varlist):
    """Read all I2C data"""

    fncname = "getI2CValues: "
    alldata = {}

    t, p, h, vis, ir = gglobs.NAN, gglobs.NAN, gglobs.NAN, gglobs.NAN, gglobs.NAN

    while not gglobs.QueueBME280.empty():
        latestReading = gglobs.QueueBME280.get()
        wprint (fncname + "Queue  B    latestReading:{}".format(latestReading))
        t, p, h = latestReading[1:]

    while not gglobs.QueueTSL2591.empty():
        latestReading = gglobs.QueueTSL2591.get()
        wprint (fncname + "Queue  T    latestReading:{}".format(latestReading))
        vis, ir, visraw, irraw, gainFct, inttime = latestReading[1:]

    #print("t:{}, p:{}, h:{}, vis:{}, ir:{}".format(t, p, h, vis, ir))

    for vname in varlist:
        if   vname in ("T"):
            vt                  = round(scaleVarValues(vname, t,   gglobs.ValueScale[vname]), 2)
            alldata.update(       {vname: vt})

        elif vname in ("P"):
            vp                  = round(scaleVarValues(vname, p,   gglobs.ValueScale[vname]), 2)
            alldata.update(       {vname: vp})

        elif vname in ("H"):
            vh                  = round(scaleVarValues(vname, h,   gglobs.ValueScale[vname]), 2)
            alldata.update(       {vname: vh})

        elif vname in ("X"):
            vvis                = round(scaleVarValues(vname, vis, gglobs.ValueScale[vname]), 2)
            alldata.update(       {vname: vvis})

        elif vname in ("CPS3rd"):
            vir                 = round(scaleVarValues(vname, ir , gglobs.ValueScale[vname]), 2)
            alldata.update(       {vname: vir})

    printLoggedValues(fncname, varlist, alldata)

    return alldata


def getI2CInfo(extended = False, first=False):
    """currently on ELV only"""

    if not gglobs.I2CConnection:   return "No connected device"

    if first:
        # on first call query the devices
        ELVinfo     = gglobs.elv.ELVgetInfo()
        try:
            BME280info  = bme280['hndl'].BME280getInfo()
        except:
            BME280info  = ("BME280 sensor not found", "")

        try:
            TSL2591info = tsl2591['hndl'].TSL2591getInfo()
        except:
            TSL2591info = ("TSL2591 sensor not found", "")

        gglobs.I2CDeviceDetected = ELVinfo[0]

        info  = "{:30s}{}\n"  .format("Connected Device:", "Dongle: {}".format(ELVinfo[0]))
        info += "{:30s}{}\n"  .format("Configured Variables:", gglobs.I2CVariables)
        if extended: info += "\n".join(ELVinfo[1:]) + "\n"

        info += "{:30s}{}\n".format("with Sensor:", BME280info[0])
        if extended: info += "\n".join(BME280info[1:])
        gglobs.I2CSensor1 = BME280info[0]

        info += "{:30s}{}".format("with Sensor:", TSL2591info[0])
        if extended: info += "\n".join(TSL2591info[1:])
        gglobs.I2CSensor2 = TSL2591info[0]

        vprint(info.replace("\n", "  "))
    else:
        # on 2nd and later calls do NOT query the devices, but take stored values
        # because conflict wuth serial calls
        if extended:    info = gglobs.I2CInfoExtended
        else:           info = gglobs.I2CInfo

    return info



def I2CautoBAUDRATE(usbport):
    """Tries to find a proper baudrate by testing for successful serial
    communication at up to all possible baudrates, beginning with the
    highest"""

    """
    NOTE: the device port can be opened without error at any baudrate, even
    when no communication can be done, e.g. due to wrong baudrate. Therfore we
    test for successful communication by checking for the return string
    containing 'ELV'.
    On success, this baudrate will be returned. A baudrate=0 will be returned
    when all communication fails. On a serial error, baudrate=None will be returned.
    """

    fncname = "I2CautoBAUDRATE: "

    dprint(fncname + "Autodiscovery of baudrate on port: '{}'".format(usbport))
    setDebugIndent(1)

    baudrates = gglobs.I2Cbaudrates
    baudrates.sort(reverse=True) # to start with highest baudrate
    for baudrate in baudrates:
        dprint(fncname + "Trying baudrate:", baudrate, debug=True)
        try:
            ABRser = serial.Serial(usbport, baudrate, timeout=0.5, write_timeout=0.5)
            ABRser.write(b'<y30?')
            rec = ABRser.read(140)
            #print("---------------------" + fncname + "rec: ", rec)

            while True:
                try:
                    cnt = ABRser.in_waiting
                except Exception as e:
                    exceptPrint(e, fncname + "ABRser.in_waiting Exception")
                    cnt = 0
                if cnt == 0: break
                ABRser.read(1)
                time.sleep(0.1)

            ABRser.close()
            if b"ELV" in rec:
                dprint(fncname + "Success with {}".format(baudrate), debug=True)
                break

        except Exception as e:
            errmessage1 = fncname + "ERROR: autoBAUDRATE: Serial communication error on finding baudrate"
            exceptPrint(e, errmessage1)
            baudrate = None
            break

        baudrate = 0

    dprint(fncname + "Found baudrate: {}".format(baudrate))
    setDebugIndent(0)

    return baudrate


class I2CReader(threading.Thread):
    """A simple threading class to read I2C values"""

    def __init__(self, QueueBME280, QueueTSL2591, readingDelay=1.0):
        try:
            self.readingDelay       = readingDelay  # How long to wait between reads (in sec)
            self.QueueBME280        = QueueBME280   # The queue into which BME280 readings will be placed
            self.QueueTSL2591       = QueueTSL2591  # The queue into which TSL2591 readings will be placed
            self.running            = False
            threading.Thread.__init__(self, group=None)
        except Exception as e:
            srcinfo = "I2CReader: init: "
            exceptPrint(e, srcinfo)


    def stop(self):
        """Stops this thread's activity. Note: this may not be immediate"""

        #print(ERRORCOLOR + "------------------------I2CReader: stop: invoked" + NORMALCOLOR)
        self.running = False


    def run(self):
        """invoked by starting thread"""

        print(TCYAN + "------------------------I2CReader: run: invoked" + NORMALCOLOR)
        self.running = True

        while self.running:
            #print("self.QueueBME280.qsize():", self.QueueBME280.qsize())
            try:
                if self.QueueBME280.qsize()  < 3: self.QueueBME280.put(bme280  ['hndl'].BME280getTPH())
            except Exception as e:
                srcinfo = "I2CReader: run: QueueBME280" + str(bme280)
                exceptPrint(e, srcinfo)
                self.QueueBME280.put( ("BME280", 22, 1033, 66))  # TESTING

            #print("self.QueueTSL2591.qsize():", self.QueueTSL2591.qsize())
            try:
                if self.QueueTSL2591.qsize() < 3: self.QueueTSL2591.put(tsl2591['hndl'].TSL2591getLumAuto())
            except Exception as e:
                srcinfo = "I2CReader: run: QueueTSL2591" + str(tsl2591)
                exceptPrint(e, srcinfo)
                self.QueueTSL2591.put(("TSL2591", 44, 55, 77, 88, 99, 111))  # TESTING

            time.sleep(self.readingDelay)



def printI2CDevInfo(extended=False):
    """prints basic info on the I2C device"""

    setBusyCursor()

    txt = "I2CSensors Device"
    if extended:  txt += " Extended"
    fprint(header(txt))
    fprint("Configured Connection:", "port:'{}' baud:{} timeoutR:{}s timeoutW:{}s".\
                     format(gglobs.I2Cusbport, gglobs.I2Cbaudrate, gglobs.I2Ctimeout, gglobs.I2Ctimeout_write))

    fprint(getI2CInfo(extended=extended))

    setNormalCursor()


def resetI2C():
    """Reset the ELV dongle and sensors"""

    setBusyCursor()
    fprint(header("Resetting I2C System"))

    fprint("Waiting ...")
    Qt_update()

    try:    gglobs.elv.ELVreset()                           # takes >=2 sec!
    except: edprint("I2C ELV Reset failed", debug=True)

    try:    bme280 ['hndl'].BME280Reset()
    except: edprint("I2C BME280 Reset failed", debug=True)

    try:    tsl2591['hndl'].TSL2591Reset()
    except: edprint("I2C TSL2591 Reset failed", debug=True)

    fprint("I2C Reset done")
    setNormalCursor()




#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
gdev_raspi.py - GeigerLog commands to handle the Geiger counter clicks as interrupts
                on a Raspi4 to serve as CPM/CPS input

not established - not working!
"""


###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"


from   gsup_utils       import *


def initRaspi():
    """Start the thread to monitor the audio pulses for clicks"""

    fprint("Device Raspi is not implemented", debug=True)
    return

    gglobs.RaspiLast60CPS             = np.full(60, 0) # needed???

    fncname ="initRaspi: "
    dprint(fncname + "Initialzing RaspiCounter")
    setDebugIndent(1)

    errmsg  = ""                            # what errors can be here?
    gglobs.RaspiDeviceName     = "RaspiCounter"
    gglobs.RaspiDeviceDetected = gglobs.RaspiDeviceName
    gglobs.Devices["Raspi"][0] = gglobs.RaspiDeviceDetected

    if gglobs.RaspiPulseDir     == "auto": gglobs.RaspiPulseDir     = False             # False => negative, True => positive
    if gglobs.RaspiPulseMax     == "auto": gglobs.RaspiPulseMax     = 32768             # +/- 16 bit
    if gglobs.RaspiThreshold    == "auto": gglobs.RaspiThreshold    = 60                # % of max
    if gglobs.RaspiVariables    == "auto": gglobs.RaspiVariables    = "CPM2nd, CPS2nd"
    if gglobs.RaspiCalibration  == "auto": gglobs.RaspiCalibration  = 154               # CPM/(µSv/h), = 0.065 µSv/h/CPM

    setCalibrations(gglobs.RaspiVariables, gglobs.RaspiCalibration)
    setLoggableVariables("Raspi", gglobs.RaspiVariables)

    #import RPi.GPIO as GPIO
    #~GPIO.setmode(GPIO.BOARD)

    gglobs.RaspiDevice      = sd.default.device
    gglobs.RaspiFormat      = sd.default.dtype
    gglobs.RaspiFormatText  = sd.default.dtype
    gglobs.RaspiChannels    = sd.default.channels
    gglobs.RaspiRate        = sd.default.samplerate
    #gglobs.RaspiChunk       = 32                   # in gglobs defined
    gglobs.RaspiLatency     = sd.default.latency

    dprint(fncname + "DEVICE:{}, CHANNELS:{}, RATE:{}, CHUNK:{}, FORMAT:{}, Latency:{}"\
                    .format(
                            gglobs.RaspiDevice,
                            gglobs.RaspiChannels,
                            gglobs.RaspiRate,
                            gglobs.RaspiChunk,
                            gglobs.RaspiFormat,
                            gglobs.RaspiLatency
                           )
         )

    gglobs.RaspiConnection = True

    setDebugIndent(0)

    return errmsg


def terminateRaspi():
    """Stop the Raspi counting"""

    fncname ="terminateRaspi: "

    if not gglobs.RaspiConnection: return fncname + "No RaspiCounter connection"

    gglobs.RaspiConnection = False


def sounddevThreadTarget(Dummy):
    """The thread to read the sounddev input"""

    fncname = "sounddevThreadTarget: "

    #getThreadData           = True

    gglobs.RaspiLastCps     = 0
    gglobs.RaspiLastCpm     = 0
    cps_count               = 0
    llimit                  = gglobs.RaspiThreshold / 100 * gglobs.RaspiPulseMax
    ilimit                  = 10000                             # the |pulse| must NOT exceed this value on first count in order to be countesd
                                                                # higher values give MORE counts!

    gglobs.RaspiLast60CPS   = np.full(60, 0)                    # storing last 60 sec of CPS values
    gglobs.RaspiMultiPulses = np.array([0])                     # time courses for last ~40 of the pulses
    gglobs.RaspiRecording   = np.array([0])
    nandata                 = np.full(10, gglobs.NAN)           # creating a gap of 10 values
    chunks40                = (gglobs.RaspiChunk + 10) * 40     # the length of 40 single pulses with gaps

    CHUNKstream = sd.InputStream(blocksize=gglobs.RaspiChunk)   # no callback so we get blocking read
    CHUNKstream.start()
    cstart                  = time.time()                       # to record the 1 sec collection period

    while not gglobs.RaspiThreadStop:

        #~wstart = time.time()
        try:
            record, overflowed = CHUNKstream.read(gglobs.RaspiChunk)
            npdata = record.reshape(-1)     # convert from '[ [1] [0] [3] ...[2] ]'  to '[1, 0, 3, ..., 2]'
            gglobs.RaspiRecording = np.concatenate((gglobs.RaspiRecording, npdata))[-gglobs.RaspiRate:]
        except Exception as e:
            info = fncname + "Exception reading stream "
            exceptPrint(e, info)
            npdata = np.array([0])
        #~print("npdata:  ", type(npdata), npdata[:10])
        #~wdt = (time.time() - wstart) * 1000

        #~xstart = time.time()
        if overflowed:
            # Input overflow.
            # In a stream opened with blocksize=0, indicates that data prior to
            # the first sample of the input buffer was discarded due to an overflow,
            # possibly because the stream callback is using too much CPU time.
            #
            # In a stream opened with a non-zero blocksize, it indicates that
            # data prior to one or more samples in the input buffer was discarded.
            # This can happen in full-duplex and input-only streams (including
            # playrec() and rec()).
            pass
            dprint("'overflowed' HAPPENED: Stream calls: record len:{}, overflowed:{}".format(len(record), overflowed) )

        # Determine if there is a count
        # ignore records catching pulses when already down; they were probably
        # already counted during last record
        gotCount = False
        if gglobs.RaspiPulseDir:                        # True -> positive pulse
            if npdata[0] < +ilimit and npdata.max() > +llimit:
                gotCount = True
        else:                                           # False -> negative pulse
            if npdata[0] > -ilimit and npdata.min() < -llimit:
                gotCount = True

        if gotCount:
            cps_count  += 1
            gglobs.RaspiMultiPulses   = np.concatenate((gglobs.RaspiMultiPulses, nandata, npdata))[-chunks40:]
            #~print("RaspiMultiPulses: ", len(gglobs.RaspiMultiPulses), gglobs.RaspiMultiPulses)
            #print("got count: Stream calls: {:7.3f}ms, record len:{}, overflowed:{}".format(dt, len(record), overflowed) )

        deltat = time.time() - cstart
        if deltat >= 1:
            gglobs.RaspiLastCps   = cps_count / deltat  * 1.05 # correct for calc time
            gglobs.RaspiLast60CPS = np.append(gglobs.RaspiLast60CPS, gglobs.RaspiLastCps)[1:]
            cps_count             = 0
            cstart                = time.time()

        #~xdt = (time.time() - xstart) * 1000
        #print("wdt:{:0.3f}ms, xdt:{:0.3f}ms".format(wdt, xdt))

    CHUNKstream.stop()
    CHUNKstream.close()


def getRaspiInfo(extended = False):
    """Info on settings of the sounddev thread"""

    if not gglobs.RaspiConnection:   return "No connected device"

    RaspiInfo = """Connected Device:             '{}'
Configured Variables:         {}
Geiger Tube Sensitivity:      {:0.1f} CPM/(µSv/h) ({:0.4f} µSv/h/CPM)"""\
                .format(
                        gglobs.RaspiDeviceName,
                        gglobs.RaspiVariables,
                        gglobs.RaspiCalibration, 1 / gglobs.RaspiCalibration
                       )

    if extended == True:
        RaspiInfo += """
Sampling Details:
- DEVICE:                     Input:'{}', Output:'{}'
- FORMAT:                     Input:'{}', Output:'{}'
- CHANNELS (1=Mono,2=Stereo): Input:'{}', Output:'{}'
- LATENCY [sec]:              Input:'{}', Output:'{}'
- Host API Index              {}
- RATE:                       {} (Samples per second)
- CHUNK:                      {} (Samples per read)
- Pulse Height Max            {} (System reported max signal)
- Pulse Direction             {} (negative or positive)
- Pulse Threshold             {}% of Pulse Height Max to trigger count"""\
                .format(
                        sd.default.device[0],
                        sd.default.device[1],
                        sd.default.dtype[0],
                        sd.default.dtype[1],
                        sd.default.channels[0],
                        sd.default.channels[1],
                        sd.default.latency[0],
                        sd.default.latency[1],
                        sd.default.hostapi,
                        sd.default.samplerate,

                        gglobs.RaspiChunk,
                        gglobs.RaspiPulseMax,
                        "POSITIVE" if gglobs.RaspiPulseDir else "NEGATIVE",
                        gglobs.RaspiThreshold,
                       )
        RaspiInfo += "\nFull Device Query:\n"
        RaspiInfo += str(sd.query_devices())
        RaspiInfo += "\n"

    return RaspiInfo


def toggleRaspiPulseDir():
    """toggle which pulse direction is sensed"""

    gglobs.RaspiPulseDir = not gglobs.RaspiPulseDir

    fprint(header("Switching Pulse Direction of RaspiCounter"))
    if gglobs.RaspiPulseDir:  direction = "POSITIVE"
    else:                     direction = "NEGATIVE"
    gglobs.RaspiMultiPulses = gglobs.RaspiMultiPulses[-10:]

    fprint("New pulse direction setting: {}".format(direction))


def getRaspiValues(varlist):
    """Read all sounddev data; return empty dict when not available"""

#testing
    gglobs.RaspiLastCps     = int(np.random.poisson(10, size=1))
    gglobs.RaspiLast60CPS   = np.append(gglobs.RaspiLast60CPS, gglobs.RaspiLastCps)[1:]
###

    fncname   = "getRaspiValues: "
    alldata = {}

    for vname in varlist:
        if   vname in ("CPM", "CPM1st", "CPM2nd", "CPM3rd"):
            cpm             = int(round(np.sum(gglobs.RaspiLast60CPS), 0))
            cpm             = scaleVarValues(vname, cpm, gglobs.ValueScale[vname])
            alldata.update(  {vname: cpm})

        elif vname in ("CPS", "CPS1st", "CPS2nd", "CPS3rd"):
            cps             = int(round(gglobs.RaspiLastCps, 0))
            cps             = scaleVarValues(vname, cps, gglobs.ValueScale[vname] )
            alldata.update(  {vname: cps})

    #~vprint("{:22s} Variables:{}  Data:{}  ".format(fncname, varlist, alldata))

    printLoggedValues(fncname, varlist, alldata)


    return alldata


def printRaspiDevInfo(extended=False):
    """prints basic info on the Raspi device"""

    setBusyCursor()

    txt = "Raspi Device"
    if extended:  txt += " Extended"
    fprint(header(txt))
    fprint("Configured Connection:", "None")
    rinfo = getRaspiInfo(extended=extended)
    fprint(rinfo)

    setNormalCursor()



##############################################################################
# following only relevant for use via 'main()'

def plotDataT(RaspiMultiPulses, inversrate):
    """This one used to plot either RaspiPlotSingle or RaspiMultiPulses of sound data"""

    if len(RaspiMultiPulses) == 0: return

    fncname = "plotDataT: "
    #~print(fncname + "starting with RaspiMultiPulses, inversrate: ", RaspiMultiPulses, inversrate)

    # Set "QT_STYLE_OVERRIDE" to avoid this message:
    #~  QApplication: invalid style override 'gtk' passed, ignoring it.
    #~  Available styles: Windows, Fusion
    os.environ["QT_STYLE_OVERRIDE"] = ""

    #~fig = plt.figure("Total", figsize=(14, 6))
    fig = plt.figure("Total", figsize=(14, 6), dpi=gglobs.hidpiScaleMPL)
    vprint("plotDataT: open figs count: {}, current fig: #{}".format(len(plt.get_fignums()), plt.gcf().number))

    if True:    # accumulate the counts (don't delete the figure)
        pass
    else:       # show a fresh record every time
        fig.clf()

    #print("plotDataT: RaspiMultiPulses:", len(RaspiMultiPulses), RaspiMultiPulses)
    x = np.arange(len(gglobs.RaspiMultiPulses)) * inversrate * 1000
    #~print("plotDataT: x    :", len(x), x)
    plt.xlabel("millisec")
    plt.plot(x, gglobs.RaspiMultiPulses,  'k.', linestyle='solid', linewidth=0.3, markersize=0.5)
    plt.ylim(-35000, 35000)
    plt.pause(.5)


def main():

    gglobs.debug = True
    gglobs.verbose = True
    fncname = "gdev_raspi.py: main: "
    gglobs.proglogPath          = "dummy"
    duration                    = 10   # sec of recording
    inversrate                  = 1 / gglobs.RaspiRate
    gglobs.RaspiMultiPulses     = np.zeros(256)

    gglobs.devel                = True                # to allow sound
    gglobs.RaspiActivation      = True
    gglobs.RaspiDeviceName      = "None"              # set by init
    gglobs.RaspiPulseDir        = False               # False for negative; True for positive pulse
    gglobs.RaspiThreshold       = 60                  # Percentage of pulse height to trigger a count


    print("Available Devices")
    print(sd.query_devices())
    print()

    RESET  = sd.default.reset()
    DEVICE = sd.default.device      #~DEVICE = sd.default.device = None, None --> no device!
                                    #~DEVICE = sd.default.device = 8,8        --> device 8 for output, device 8 for input
    sd.default.dtype    = ('int16', 'int16')
    sd.default.channels = (1, 1)
    FORMAT              = sd.default.dtype[1]
    CHANNELS            = sd.default.channels[1]                 # Mono; for Stereo set: #CHANNELS = 2

    #RATE = 96000
    #RATE = 44100
    #RATE = 22050
    #RATE = 11025
    RATE = sd.default.samplerate = 2**15                        # = 32768

    #CHUNK = 64
    #CHUNK = 128
    CHUNK = 256
    #CHUNK = 512
    #CHUNK = 1024
    #CHUNK = 2048
    #CHUNK = 4096


    start = time.time()
    while (time.time() - start) <= duration:
        time.sleep(.15)
        now = time.time()

        #~mystream.start()
        #~mystream.stop()
        try:
            getLongChunk(1.1)
            #print("time: {:5.3f}, gglobs.RaspiLastCps: {:5.1f}, gglobs.RaspiLastCpm: {:5.1f}".format((now -start), gglobs.RaspiLastCps, gglobs.RaspiLastCpm ))
            plotDataT(gglobs.RaspiMultiPulses, inversrate)
        except Exception as e:
            print(fncname + "Exception :", e)

    #terminateRaspi()

    plotDataT(gglobs.RaspiMultiPulses, inversrate)
    print(fncname + "Habe fertig")
    plt.show()


if __name__ == '__main__':
    import matplotlib.pyplot as plt # plotting possible only when run as main
    main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
gstat_convfft.py - GeigerLog commands for FFT statistics on convoluted functions

include in programs with:
    import gstat_convfft
"""

###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"


from   gsup_utils            import *


def convFFT():
    """Plotting FFT and Autocorrelation after connvolution of time functions
    t       = time
    sigt    = Signal in time domain, (CPM/CPS here)
    freq    = Signal in frequency domain

    Has extra function for rectangle and autocorr
    """

    markersize     = 1.0
    rectangle_size = 60     # normally 60 for 60 s = 1 min

    vindex      = gglobs.exgg.select.currentIndex()
    vname       = list(gglobs.varsBook)[vindex]
    vnameFull   = gglobs.varsBook[vname][0]
    yunit       = vnameFull
    vprint("plotFFT: vname: '{}', vnameFull: '{}'".format(vname, vnameFull))

    rawt0    = gglobs.logTimeDiffSlice
    rawsigt0 = gglobs.logSliceMod[vname]

    if rawsigt0 is None:
        showStatusMessage("No data available")
        return

    if rawt0.size <= rectangle_size:
        showStatusMessage("Not enough data (need more than {})".format(rectangle_size))
        return

    setBusyCursor()
    DataSrc = os.path.basename(gglobs.currentDBPath)

    rawt    = np.ndarray(0)
    rawsigt = np.ndarray(0)
    for i in range(0, len(rawt0)):
        if np.isnan(rawsigt0[i]):
            #print("i, x0[i]:", i, x0[i])
            continue
        else:
            #print("i, x[i]:", i, x0[i])
            rawt    = np.append(rawt,    rawt0[i])
            rawsigt = np.append(rawsigt, rawsigt0[i])
    #print("rawt, rawsigt: len:", len(rawt), len(rawsigt))

    t    = rawt.copy()
    sigt = rawsigt.copy()

    """Call numpy.isnan(arr) to get a boolean array showing whether or not each
    index in the initial array arr has a value of NaN. Use the ~ operator to
    invert this array so that indices with NaN are now marked as False. Then
    call indexing syntax arr[n_arr] with n_arr as the result of the last step
    to get a new array with all NaNs filtered out."""
    nan_array = np.isnan(sigt)
    not_nan_array = ~ nan_array
    sigt = sigt[not_nan_array]

    if sigt.size == 0:
        gglobs.exgg.showStatusMessage("No data available")
        setNormalCursor()
        return


# Window functions ############################################################
    # the only place to activate Window function is here
    use_window_functions = False

    if use_window_functions:
        hamm    = np.hamming (len(t))
        hann    = np.hanning (len(t))
        black   = np.blackman(len(t))
        # Kaiser:
        # "A beta value of 14 is probably a good starting point"
        # beta  Window shape
        # 0     Rectangular
        # 5     Similar to a Hamming
        # 6     Similar to a Hanning
        # 8.6   Similar to a Blackman
        beta    = 5
        kaiser  = np.kaiser(len(t), beta)

        # Select one
        #win     = hamm
        #win     = hann
        #win     = black
        win     = kaiser

        # When using window functions subtract the average in order to avoid
        # spurious low-frequency peaks!
        sigt   = sigt - np.mean(sigt)

        # Time domain signal with Window function applied
        sigt_win = sigt * win

# Prepare variables ###########################################################

    t               = t * 1440.0  # convert days to minutes
    timeunit        = "minutes"
    frequencyunit   = "1/minute"
    cycletime       = (t[-1] - t[0]) / (t.size -1)  # in minutes

    sigt_mean       = np.mean(sigt)
    sigt_var        = np.var(sigt)
    sigt_std        = np.std(sigt)
    sigt_var        = np.var(sigt)
    sigt_err        = sigt_std / np.sqrt(sigt.size)

    print("t:    size:"       , t.size        , "\n", t[:30])
    print("sigt: size:"       , sigt.size     , "\n", sigt[:30])
    #print("sigt_win: size:"  , sigt_win.size , "\n", sigt_win[:30])

    if t.size < rectangle_size:
        msg = "Not enough data; need {} records as minimum".format(rectangle_size)
        gglobs.exgg.showStatusMessage(msg)
        setNormalCursor()
        return

    maxf = int(sigt.size / 60 * 10) # --> up to 10/min frequency

# figure and canvas ###########################################################
    figEvalFFT = plt.figure(facecolor = "#C9F9F0", dpi=gglobs.hidpiScaleMPL) # blueish tint
    vprint("figEvalFFT: open figs count: {}, current fig: #{}".format(len(plt.get_fignums()), plt.gcf().number))

# arrange sub plots
    plt.subplots_adjust(hspace=0.4, wspace=0.3, left=.04, top=0.93, bottom=0.08, right=.99)

    # canvas - this is the Canvas Widget that displays the `figure`
    # it takes the `figure` instance as a parameter to __init__
    canvas3 = FigureCanvas(figEvalFFT)
    canvas3.setFixedSize(1800, 700)
    navtoolbar = NavigationToolbar(canvas3, None)

# Data vs Time ################################################################
    plt.subplot (2,4,2)
    plt.title   ("Time (Counts)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(sigt.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel  ("Time ({})".format(timeunit), fontsize=12)
    plt.ylabel  ("Count Rate  " + yunit, fontsize=12)
    plt.grid    (True)
    plt.ticklabel_format(useOffset=False)

    plt.plot    (t, sigt        ,  linewidth=0.4, color='red'   , label ="Time Domain" , marker="o", markeredgecolor='red'   , markersize=markersize)

# Autocorrelation vs Lag ######################################################
    # calculations
    asigt = sigt - sigt_mean
    #print "np.mean(sigt) , np.var(sigt) :", np.mean(sigt),  np.var(sigt)
    #print "np.mean(asigt), np.var(asigt):", np.mean(asigt), np.var(asigt)

    asigtnorm = np.var(asigt) * asigt.size  # to normalize autocorrelation
    ac = np.correlate(asigt, asigt, mode='full') / asigtnorm
    ac = ac[int(ac.size/2):]
    #print "ac: len:", ac.size
    #print "ac:", "\n", ac

    # autocorrelation plot
    aax1 =  plt.subplot(2, 4, 1)

    plt.title   ("Autocorrelation (normalized) vs. Lag Period", fontsize=11, loc = 'left', y = 1.08)
    plt.xlabel  ("Lag Period ({})".format(timeunit), fontsize=12)
    plt.ylabel  ("Autocorrelation", fontsize=12)
    plt.grid    (True)
    #plt.ticklabel_format(useOffset=False)

    aax2 = aax1.twiny()

    # how many points to show enlarged?
    for i in range(t.size):
        if ac[i] < 0: break

    tindex = min(i, t.size * 0.01)
    tindex = max(25, tindex, 60./(cycletime * 60.))
    tindex = int(tindex)  # Warning: ./geigerlog:3483: VisibleDeprecationWarning: using a non-integer number instead of an integer will result in an error in the future
                          # aax2.plot(tnew[:tindex], ac[:tindex], linewidth= 2.0, color='blue' , label ="Expanded Lag Period - Top Scale" , marker="o", markeredgecolor='blue'  , markersize=markersize*2)
                          # What is the reason ?????
    #print "tindex:", tindex

    tnew = t - t[0]
    aax1.plot(tnew,          ac         , linewidth= 1.0, color='red'  , label ="Full Lag Period - Bottom Scale" , marker="o", markeredgecolor='red'   , markersize=markersize * 1)
    #aax1.legend(loc='upper right', fontsize=12)

    aax2.plot(60*tnew[:tindex], ac[:tindex], linewidth= 2.0, color='blue' , label ="Expanded Lag Period in sec - Top Scale" , marker="D", markeredgecolor='blue'  , markersize=markersize * 1)
    #print "ac:", ac[:10]

    plt.legend(loc='upper right', fontsize=8) # larger does not fit

    for a in aax1.get_xticklabels():
        #a.set_color("red")
        #a.set_weight("bold")
        pass

    for a in aax2.get_xticklabels():
        a.set_color("blue")
        #~a.set_weight("bold")

# FFT plots ###################################################################
    # calculations
    # using amplitude spectrum, not power spectrum; power would be freq^2
    freq                = np.abs(np.fft.rfft(sigt     ))
    #freq2              = np.abs(np.fft.rfft(sigt2    ))
    print("freq:"       , len(freq)     , "\n", freq[0:25])

    if use_window_functions:
        freq_win        = np.abs(np.fft.rfft(sigt_win))
        print("freq_win:"   , len(freq_win) , "\n", freq_win[0:25])

    f = np.fft.rfftfreq(t.size, d = cycletime)
    print("f:   len:", f.size, "\n", f[0:25])

    p  = np.reciprocal(f[1:])  # skipping 1st value frequency = 0
    print("Period: len:", p.size, "\n", p[0:25])


    # Plot FFT vs Time #########################################################
    plt.subplot(2, 4, 5)
    plt.title("FFT (Counts)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(freq.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel("Time Period ({})".format(timeunit), fontsize=12)
    plt.ylabel("FFT Amplitude", fontsize=12)
    plt.grid(True)
    plt.ticklabel_format(useOffset=False)

    plt.loglog(p, freq[1:]        , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)

    # Plot FFT vs Frequency ####################################################
    plt.subplot(2, 4, 6)
    plt.title("FFT (Counts)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(freq.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel("Frequency ({})".format(frequencyunit), fontsize=12)
    plt.ylabel("FFT Amplitude", fontsize=12)
    plt.grid(True)
    plt.ticklabel_format(useOffset=False)

    #~plt.semilogy (f[1:], freq[1:]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)
    plt.semilogy (f[1:maxf], freq[1:maxf]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)


# convolution plots ###########################################################

    # rect for convolution
    nr = rectangle_size # nr values of 1, followed by zeros

    rect = np.zeros(sigt.size)
    for i in range(nr): rect[i] = 1
    print("rect:", len(rect), ", Values:\n0:10: ", rect[0:10], "\n50:70:", rect[50:70])

    # time axis
    #~bf = t[:rect.size]


# Plot Rectangle Signal vs time ###############################################
    plt.subplot (2, 4, 3)
    plt.title   ("Time (Rectangle)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(rect.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel  ("Time ({})".format(timeunit), fontsize=12)
    plt.ylabel  ("Signal Value", fontsize=12)
    plt.grid    (True)
    plt.ticklabel_format(useOffset=False)
    #~plt.plot (bf, rect     , linewidth= 1.0, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)
    plt.plot (t[:rect.size], rect     , linewidth= 1.0, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)


# FFT of Signal vs Frequency ##################################################

    cfreq = np.abs(np.fft.rfft(rect     ))
    print("cfreq:   len:", cfreq.size, "\n", cfreq[0:25])

    f = np.fft.rfftfreq(t.size, d = cycletime)
    print("f:   len:", f.size, "\n", f[0:25])

    p  = np.reciprocal(f[1:])  # skipping 1st value frequency = 0
    print("Period: len:", p.size, "\n", p[0:25])

    plt.subplot (2, 4, 7)
    plt.title   ("FFT (Rectangle)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(cfreq.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel  ("Frequency ({})".format(frequencyunit), fontsize=12)
    plt.ylabel  ("FFT Amplitude", fontsize=12)
    plt.grid    (True)
    plt.ticklabel_format(useOffset=False)

    #~plt.semilogy (f[1:], cfreq[1:]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)
    plt.semilogy (f[1:maxf], cfreq[1:maxf]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)


# last columns (convolution columns) ##########################################
    # upper

    csigt = scipy.signal.convolve(rect, sigt ) * (60 / nr)
    csigt = csigt[nr:len(sigt) + nr]
    print("csigt:", len(csigt), csigt[:30])

    plt.subplot (2, 4, 4)
    plt.title   ("Time (Counts && Rectangle)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(csigt.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')

    plt.xlabel  ("Time ({})".format(timeunit), fontsize=12)
    plt.ylabel  ("Count Rate CPM", fontsize=12)
    plt.grid    (True)
    plt.ticklabel_format(useOffset=False)

    plt.plot (t[:-nr], csigt[:-nr]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)

    # lower
    ccfreq = cfreq * freq

    plt.subplot (2, 4, 8)
    plt.title   ("FFT (Counts && Rectangle)", fontsize=12, loc = 'left')
    subTitle = "Recs:" + str(cfreq.size)
    plt.title   (subTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel  ("Frequency ({})".format(frequencyunit), fontsize=12)
    plt.ylabel  ("FFT Amplitude", fontsize=12)
    plt.grid    (True)
    plt.ticklabel_format(useOffset=False)

    #~plt.semilogy (f[1:], ccfreq[1:]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)
    plt.semilogy (f[1:maxf], ccfreq[1:maxf]     , linewidth= 0.4, color='red'   , label ="FFT" , marker="o", markeredgecolor='red'   , markersize=markersize)


# textboxes ###################################################################
    labout_left  = QTextBrowser() # label to hold some data on left side
    labout_left.setLineWrapMode(QTextEdit.NoWrap)
    labout_left.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)
    labout_left.setMinimumHeight(150)

    labout_left.append("{:22s}= {}"                             .format('File'               , DataSrc))
    labout_left.append("{:22s}= {}"                             .format("No of Records"      , t.size))
    labout_left.append("{:22s}= {:4.2f}"                        .format("Count Rate Average" , sigt_mean))
    labout_left.append("{:22s}= {:4.2f} (Std.Dev:{:5.2f}, Std.Err:{:5.2f})"    .format("Count Rate Variance" , sigt_var, sigt_std, sigt_err))
    labout_left.append("{:22s}= {:4.2f} sec (overall average)"  .format("Cycle Time"         , cycletime * 60.)) # t is in minutes
    labout_left.append("{:22s}= {:4.2f} "                       .format("A.corr(lag=  0   sec)", ac[0]))
    labout_left.append("{:22s}= {:4.2f} "                       .format("A.corr(lag={:5.1f} sec)".format(tnew[1] *60.), ac[1]))
    labout_left.append("{:22s}= {:4.2f} "                       .format("A.corr(lag={:5.1f} sec)".format(tnew[2] *60.), ac[2]))
    labout_left.append("{:22s}= {:4.2f} "                       .format("A.corr(lag={:5.1f} sec)".format(tnew[3] *60.), ac[3]))

    labout_right  = QTextBrowser() # label to hold some data on right side
    labout_right.setLineWrapMode(QTextEdit.NoWrap)
    labout_right.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)
    labout_right.setMinimumHeight(120)

    fftmax      = np.max    (freq[1:])
    fftmaxindex = np.argmax (freq[1:]) + 1
    f_max       = f         [fftmaxindex ]

    labout_right.append("{:22s}: {:s}"                 .format("Legend", "'Counts && Rectangle' means: 'Counts convolved with Rectangle'\n"))
    labout_right.append("{:22s}= {:4.0f}"              .format("FFT(f=0)"         , freq[0]) )
    labout_right.append("{:22s}= {:4.0f}"              .format("len(t)"           , len(t)) )
    labout_right.append("{:22s}= {:4.2f} (= FFT(f=0)/No of Records)".format("Count Rate Average", freq[0] / len(t)) )
    labout_right.append("{:22s}= {:4.2f}"              .format("Max FFT(f>0)"     , fftmax))
    labout_right.append("{:22s}= {}"                   .format("  @ Index"        , fftmaxindex))
    labout_right.append("{:22s}= {:4.4f}"              .format("  @ Frequency"    , f_max ))
    try:
        labout_right.append("{:22s}= {:4.4f}"          .format("  @ Period"       , p[fftmaxindex] ))
    except:
        labout_right.append("{:22s}= {:s}"             .format("  @ Period"       , "undefined" ))


# Pop Up  #################################################################
    d       = QDialog()
    d.setWindowIcon(gglobs.iconGeigerLog)
    d.setFont(gglobs.fontstd)
    d.setWindowTitle("FFT & Autocorrelation")
    #d.setMinimumHeight(gglobs.window_height)
    #d.setWindowModality(Qt.ApplicationModal)
    #d.setWindowModality(Qt.NonModal)
    d.setWindowModality(Qt.WindowModal)

    bbox    = QDialogButtonBox()
    bbox.setStandardButtons(QDialogButtonBox.Ok)
    bbox.accepted.connect(lambda: d.done(0))

    layoutH   = QHBoxLayout()
    layoutH.addWidget(labout_left)
    layoutH.addWidget(labout_right)

    layoutV   = QVBoxLayout(d)
    layoutV.addWidget(navtoolbar)
    layoutV.addWidget(canvas3)
    layoutV.addLayout(layoutH)
    layoutV.addWidget(bbox)

    figEvalFFT.canvas.draw_idle()
    d.exec_()
    plt.close(figEvalFFT)



#! /usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
Code for the ELV USB-I2C Dongle
(adapted from file: I2Cpytools_dgl_ELV.py)
"""

###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"

from   gsup_utils       import *

try:
    import serial                       # serial port (module has name: 'pyserial'!)
    import serial.tools.list_ports      # allows listing of serial ports
except Exception as e:
    msg  = "\nPython module 'serial' was not found\n"
    msg += "In order to run any I2C device this module is required\n"
    msg += "Verify that 'pyserial' is installed using GeigerLog tool: 'gtools/GLpipcheck.py'"
    exceptPrint(e, msg)
    edprint("Halting GeigerLog")
    playWav("err")
    sys.exit()


class ELVdongle:
    """Code for the ELV USB-I2C Dongle"""

    #              ELV name    xX   time   reqB   wrt/rec   info     rec
    pTemplate   = "ELV {:7s} {:2s} {:10s} [{:3d}] [{:3d}]  {:20s} == {}"

    def __init__(self):
        """make instance of dongle"""

        self.name        = "ELVdongle"
        self.short       = "dongle"


    def ELVinit(self):
        """opening the serial port and testing for correct dongle"""

        # open serial port
        msg = "port: {} (baudrate:{}, timeout:{}s, timeout_write:{}s)"\
                  .format(gglobs.I2Cusbport, gglobs.I2Cbaudrate, gglobs.I2Ctimeout, gglobs.I2Ctimeout_write)
        try:
            gglobs.I2Cser = serial.Serial(gglobs.I2Cusbport, gglobs.I2Cbaudrate, timeout=gglobs.I2Ctimeout, write_timeout=gglobs.I2Ctimeout_write)
            dprint("ELV dongle opened at " + msg)
        except Exception as e:
            exceptPrint(e, "ERROR Serial port for ELV dongle could not be opened")
            return False, msg

        # stop macro with '<' and set 'y30' in order to NOT get ACK and NACK
        # and then get info by '?', all in single command
        self.ELVwriteAdmin(b'<y30?', name=self.short, info="Init Dongle")
        rec = self.ELVreadAdmin(140, name=self.short, info="").strip()
        if rec.startswith(b"ELV"):  return True, ""                                         # an ELV dongle
        else:                       return False, "Not an ELV dongle!<br>at {}".format(msg) # not an ELV dongle


    def ELVreset(self):
        """Reset the ELV dongle (takes 2 sec!)"""

        #print("\n---- Reset")
        self.ELVwriteAdmin(b'z4b', name=self.short, info="Reset Dongle")
        time.sleep(2) # 1 sec is NOT enough
        rec = self.ELVreadAdmin(name=self.short, info="")
        #print(rec.decode("utf-8") )


    def ELVgetInfo(self):
        """Show the startup info"""

        self.ELVwriteAdmin(b'?', name=self.short, info="Show Info")
        rec = self.ELVreadAdmin(140, name=self.short, info="").strip()

        return rec.decode("utf-8").split("\r\n")


    def ELVgetMacro(self):
        """ Print the Macro in Memory """

        self.ELVwriteAdmin(b'U', name=self.short, info="Show Macrocode" )
        rec = self.ELVreadAdmin(255+26, name=self.short, info="")
        rd = (rec.decode("utf-8")).split("\r\n")

        return rec.decode("utf-8")


    def ELVwriteAdmin (self, command, name= "-", info = "no info"):
        """write to the ELV USB-I2C - only for its interal admin functions"""

        command = command.upper()
        wrt     = gglobs.I2Cser.write(command)  # wrt = no of bytes written
    #    print( self.pTemplate.format(name, "TA", stime()[11:], len(command), wrt, info, command))


    def ELVreadAdmin (self, length = 100, name = "-", info = "no info"):
        """read from the ELV USB-I2C - only for its internal admin functions"""

        fncname = "ELVreadAdmin: "

        rec = gglobs.I2Cser.read(length)
        try:
            cnt = gglobs.I2Cser.in_waiting
        except Exception as e:
            exceptPrint(e, fncname + "I2Cser.in_waiting Exception")
            cnt = 0

        if cnt > 0:
            print("ELVreadAdmin: Bytes waiting:", cnt)
            start = time.time()
            while True and (time.time() - start) < gglobs.I2Ctimeout:            # read single byte until nothing is returned
                x = gglobs.I2Cser.read(1)
                if len(x) == 0: break
                rec += x
    #    print("ELVreadAdmin: len(rec), rec:", len(rec), rec)
        rec = rec.strip()
    #    print( self.pTemplate.format(name, "RA", stime()[11:], length, len(rec), info, rec))

        return rec


    def ELVaskDongle(self, addr, data, rbytes, name="no name", info="no info", doPrint=True, end="\n"):
        """ Function doc """

        doPrint = False

        self.ELVwriteData(addr, data, name=name, info=info, doPrint=doPrint)
        if rbytes > 0:
            self.ELVinitializeRead(addr, rbytes, name=name, doPrint=doPrint)
            answ = self.ELVreadData(length=rbytes, name=name, doPrint=doPrint)
            if doPrint: print(end=end)
        else:
            answ = None

        return answ


    def ELVwriteData (self, addr, data, name="", info ="", doPrint=True):
        """write commands for the sensors via the ELV USB-I2C"""

        doPrint = False

        wA      = "{:02X}".format((addr << 1) + 0)
        wdata   = ""
        for a in data: wdata += " {:02X}".format(a) # puts a space at the beginning

        command = bytes('S {}{} P'.format(wA, wdata), 'ASCII').upper()
        wrt     = gglobs.I2Cser.write(command)  # wrt = no of bytes written
        if doPrint: print(self.pTemplate.format(name, "TX", stime()[11:], len(command), wrt, info, command))


    def ELVinitializeRead(self, addr, rbytes, name="", info="", doPrint=True):
        """initialize reading from the sensor via the ELV USB-I2C"""

        doPrint = False

        rA      = "{:02X}".format((addr << 1) + 1)  # read address
        rb      = "{:02X}".format(rbytes)           # no of bytes
        command = bytes('S {} {} P'.format(rA, rb), 'ASCII').upper()
        wrt     = gglobs.I2Cser.write(command)  # wrt = no of bytes written
        if doPrint: print(self.pTemplate.format("", "iR", stime()[11:], len(command), wrt, info, command))


    def ELVreadData (self, length=1, name="", info="", doPrint=True):
        """ read data from the ELV USB-I2C """

        fncname = "ELVreadData: "

        doPrint=False

        rec = gglobs.I2Cser.read(length * 3 + 2) # 3 chars per byte('FF ') + CR, LF
        try:
            cnt = gglobs.I2Cser.in_waiting
        except Exception as e:
            exceptPrint(e, fncname + "I2Cser.in_waiting Exception")
            cnt = 0

        if cnt > 0:
            print("ELVreadData Bytes waiting:", cnt)
            while True:                # read single byte until nothing is returned
                x = gglobs.I2Cser.read(1)
                if len(x) == 0: break
                rec += x

        rec = rec.rstrip(b"\r\n")       # remove carridge return, linefeed, keep last space

        if doPrint: print( self.pTemplate.format("", "RX", stime()[11:], length, len(rec), info, rec), end="")

        if not rec.strip().startswith(b"Solve ") or \
           not rec.strip().startswith(b"Err: "):        # conversion unless an error msg from dongle
            rlist = self.__getListfromRec(rec)
            return rlist
        else:
            return rec


    def ELVclose(self):
        """ Close the serial port """

        gglobs.I2Cser.close()
        wprint("ELV is closed")


    def __getListfromRec(self, rec):
        """ convert ASCII string from terminal to list of integer"""
        # rec    : b'BE 6E 9A 69 32 00 ...'
        # returns: [190, 110, 154, 105, 50, 0, ...]

        print("rec: ", rec)
        rlist = []
        if len(rec) >= 3:
            for i in range(0, len(rec), 3):
                try:
                    rlist.append(int(rec[i:i+3], 16))
                except Exception as e:
                    msg = "__getListfromRec: rec: {}".format(rec)
                    exceptPrint(e, msg)

        else: # empty record
            pass

        return rlist




#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
gstat_poisson.py - GeigerLog commands for poisson statistics

include in programs with:
    import gstat_poisson
"""

###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"


from   gsup_utils            import *


def plotPoisson():
    """Plotting a Poisson Fit to a histogram of the data"""

    setBusyCursor()

    vindex      = gglobs.exgg.select.currentIndex()
    vname       = list(gglobs.varsBook)[vindex]
    vnameFull   = gglobs.varsBook[vname][0]

    if gglobs.logTimeDiffSlice is None:
        gglobs.exgg.showStatusMessage("No data available")
        setNormalCursor()
        return

    try:
        t0 = gglobs.logTimeDiffSlice
        x0 = gglobs.logSliceMod[vname]
    except Exception as e:
        gglobs.exgg.showStatusMessage("No data available")
        setNormalCursor()
        return
    #print("t0, x0: len:", len(t0), len(x0))

    fncname = "plotPoisson: "
    vprint(fncname + "Plotting Histogram and Poisson Fit")
    setDebugIndent(1)

    # elimitate all nan data in x (t will always exist)
    counter_isnan = 0
    t = np.ndarray(0)
    x = np.ndarray(0)
    for i in range(0,len(t0)):
        #print("i, x0[i]:", i, x0[i])
        if np.isnan(x0[i]):
            counter_isnan += 1
            pass
        else:
            t = np.append(t, t0[i])
            x = np.append(x, x0[i])
    #~print("len(t0), len(x0), len(t), len(x): ", len(t0), len(x0), len(t), len(x))
    #~if counter_isnan > 0:   print("Found nan, count:", counter_isnan)
    #~else:                   print("Found no nan")

    if t.size == 0:
        gglobs.exgg.showStatusMessage("No data available")
        setNormalCursor()
        setDebugIndent(0)
        return

    DataSrc     = os.path.basename(gglobs.currentDBPath)
    cycletime   = (t[-1] - t[0]) / (t.size - 1)  # in minutes
    yunit       = vnameFull

    ######################################
    # to show histogram of delta between 2 consecutive counts
    if 0:       # do not execute on 0
        dx = x[:-1].copy()
        for i in range(0, len(dx)):
            dx[i] = abs(x[i+1] - x[i])
            #if dx[i] > 10: print i, dx[i]
        #print x, len(x)
        #print dx, len(dx)
        x = dx
        yunit = "Differences between 2 consecutive CPM!"
    #######################################


    lenx        = len(x)
    sumx        = np.nansum (x)         # nan...: all values excluding NANs!
    avgx        = np.nanmean(x)         # though there shouldn't be any, as they
    varx        = np.nanvar (x)         # are eliminated above
    stdx        = np.nanstd (x)         # But it does not hurt
    minx        = np.nanmin (x)
    maxx        = np.nanmax (x)
    if avgx >= 0:
        std95   = np.sqrt(avgx) * 1.96  # +/- std95 is range for 95% of all values
    else:
        std95   = gglobs.NAN

    if avgx == 0:
        gglobs.exgg.showStatusMessage("All Variable data are zero; cannot calculate Poisson distribution!")
        setDebugIndent(0)
        setNormalCursor()
        return


    wprint(fncname + "count data: lenx:{}, sumx:{:5.0f}, avgx:{:5.3f}, varx:{:5.3f}, stdx:{:5.3f}, minx:{:5.3f}, maxx:{:5.3f}, std95%:{:5.3f}\n{}\n".\
                format(lenx,   sumx,         avgx,          varx,         stdx,         minx,         maxx,         std95,           x))

    # take the lower of (the lowest count rate) and (the average minus 2 StdDev), but must be at least zero
    bin_center_min    = int(max(0,    min(minx , avgx - (std95 * 2))))

    # take the higher of (the highest count rate) and (the average plus 2 StdDev) and 16
    bin_center_max    = int(max(16, maxx , avgx + (std95 * 2)))

    # limit the total no of bins to 30 by making the bins wider, but keep width at least at 1
    step              = int(max(1, int((bin_center_max - bin_center_min) / 30)))
    bin_total         = int((bin_center_max - bin_center_min) / step) + 1
    #print("  step: {}, bin_center_min: {}, bin_center_max: {}, bin_total: {}".format(step, bin_center_min, bin_center_max, bin_total))

    bins    = np.empty(bin_total + 1)
    bins[0] = int(bin_center_min)
    for i in range(1, bin_total + 1):
        bins[i] = int(bins[i - 1] + step)
    #print("bins: len(): ", len(bins), bins)


    # CREATE histogram
    # https://numpy.org/devdocs/reference/generated/numpy.histogram.html
    # If bins is an int, it defines the number of equal-width bins in the given
    # range (10, by default).If bins is a sequence, it defines the bin edges,
    # including the rightmost edge, allowing for non-uniform bin widths.
    # If bins is a string, it defines the method used to calculate the optimal
    # bin width, as defined by histogram_bins.
    # hist, bins = np.histogram(x, bins='auto') # gives bins as rational numbers
    # hist, bins = np.histogram(x, bins='sqrt') # used in Excel; bins as rational numbers
    #
    # returns: The values of the histogram. See density and weights for a
    #          description of the possible semantics.
    #          bins : array of dtype float. Return the bin edges (length(hist)+1).

    # Here using manually created histogram, as otherwise a synthetic normal distribution
    # would not properly sum up
    hist = np.empty( len(bins) - 1 )
    for i in range(0, len(bins) - 1):
        stepsum = 0
        ll0 = bins[i ]
        hl0 = bins[i + 1]
        dl0 = hl0 - ll0
        for j in range(0, step):
            ll = ll0 - (dl0 / 2 / step) + dl0 /step * j
            hl = ll + dl0 / step
            stepsum += len( x[((x>=ll) & (x<hl))] )
            #print("i, j, ll0, hl0,  ll, hl, stepsum: ", i, j, ll0, hl0, ll, hl, stepsum)
        hist[i] = stepsum
    #print( "manual histogram: len(hist):", len(hist), "\n", hist)



    # sum up the Poisson dist for the bins from above histogram
    pdfs = []
    for i in range(int(bins[0]), int(bins[-1]), int(step)):
        stepsum = 0
        for j in range(0, step):
            stepsum += scipy.stats.poisson.pmf(i + j, avgx)
        pdfs.append(stepsum * lenx)

    #print("----------------avgx = ", avgx)
    pdfnorm = []
    for i in range(int(bins[0]), int(bins[-1]), int(step)):
        stepsum = 0
        for j in range(0, step):
            if avgx >= 0:
                stepsum += scipy.stats.norm.pdf(i + j , avgx, scale=np.sqrt(avgx))
            else:
                stepsum += scipy.stats.norm.pdf(i + j , avgx, scale=gglobs.NAN)
        pdfnorm.append(stepsum * lenx)

    # determine r-squared for Poisson
    ss_res = np.sum((hist - pdfs    ) ** 2)         # residual sum of squares
    ss_tot = np.sum((hist - np.mean(hist)) ** 2)    # total sum of squares
    r2 = 1 - (ss_res / ss_tot)                      # r-squared for poisson

    # determine r-squared for Normal
    ss_res = np.sum((hist - pdfnorm    ) ** 2)      # residual sum of squares
    ss_tot = np.sum((hist - np.mean(hist)) ** 2)    # total sum of squares
    r2N = 1 - (ss_res / ss_tot)                     # r-squared for normal



# chi squared stuff  ----------------------------------------------------------
    obs     = hist
    exp     = pdfs
    mini    = 0
    maxi    = len(obs)

    # find where obs and exp are both > 5
    # first the left side
    for i in range(len(obs)):
        #print("Left:   i={}, obs={:9.0f}, exp={:9.2f}".format(i, obs[i], exp[i] ))
        if obs[i] >=5 and exp[i] >= 5:
            #print("mini--> i={}, obs= {}, exp={}".format(i, obs[i], exp[i]))
            mini = i
            break

    # now the right side
    for i in range(mini, len(obs) ):
        #print("Right: i={}, obs={:9.0f}, exp={:9.2f}".format(i, obs[i], exp[i] ))
        if obs[i] <= 5 or exp[i] <= 5:
            #print("maxi--> i={}, obs= {}, exp={}".format(i, obs[i], exp[i]))
            maxi = i
            break

    # the ignored values on the right side
    for i in range(maxi, len(obs) ):
        pass
        #print("Rest:  i={}, obs={:9.0f}, exp={:9.2f}".format(i, obs[i], exp[i] ))

    wprint(fncname + "mini:{}, maxi:{}, diff:{}".format(mini, maxi, maxi - mini))



# calc chi2 manually
    #obs_mima = obs[mini:maxi] # cut out the part where obs and exp are both > 5
    #exp_mima = exp[mini:maxi]
    #
    #sumchi2 = 0
    #for i in range(0, len(obs_mima)):
    #    v = (obs_mima[i] - exp_mima[i])**2/exp_mima[i]
    #    sumchi2 += v
    #    print("i={:4d}, obs={:11.4f}, exp={:11.4f}, obs-exp={:11.4f}, chi2={:11.4f}, sumchi2={:11.4f}".format(i, obs_mima[i], exp_mima[i], obs_mima[i] - exp_mima[i], v, sumchi2))

#testing full Hist, not selected for > 5
    # calc chi2 for Poisson
    #ddofPoiss               = 1
    #dofPoiss                = len(hist) - ddofPoiss
    #chi2Poiss, pchi2Poiss   = scipy.stats.chisquare(hist, f_exp=pdfs,    ddof=ddofPoiss, axis=None)
    #txtChi2Poiss            = "Chi-squared Test Poisson <5:  DoF= {:1d}, chi² = {:5.1f}, p0= {:2.1%}".format(dofPoiss, chi2Poiss, pchi2Poiss)
    #vprint(fncname + txtChi2Poiss)


    # Degrees of Freedom
    # https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.chisquare.html
    #   "ddofint, optional:
    #   Delta degrees of freedom”: adjustment to the degrees of freedom for the
    #   p-value. The p-value is computed using a chi-squared distribution with
    #   k - 1 - ddof degrees of freedom, where k is the number of observed
    #   frequencies. The default value of ddof is 0."
    #
    # assumption: for Poisson take 1 extra dof off ddof= 1 (estimate average from data)
    #             for Normal  take 2 extra dof off ddof= 2 (estimate average + StdDev from data)
    #
    # calc chi2 for Poisson
    ddofPoiss               = 1
    dofPoiss                = len(hist[mini:maxi]) - ddofPoiss
    chi2Poiss, pchi2Poiss   = scipy.stats.chisquare(hist[mini:maxi], f_exp=pdfs[mini:maxi],    ddof=ddofPoiss, axis=None)
    # testing same fucntion gives p=100%
    #chi2Poiss, pchi2Poiss   = scipy.stats.chisquare(pdfs[mini:maxi], f_exp=pdfs[mini:maxi],    ddof=ddofPoiss, axis=None)
    txtChi2Poiss            = "Chi-squared Test Poisson:  DoF = {:1d}, chi² = {:5.1f}, p = {:2.1%}".format(dofPoiss, chi2Poiss, pchi2Poiss)
    vprint(fncname + txtChi2Poiss)

    # calc chi2 for Normal
    ddofNorm                = 2
    dofNorm                 = len(hist[mini:maxi]) - ddofNorm
    chi2Norm, pchi2Norm     = scipy.stats.chisquare(hist[mini:maxi], f_exp=pdfnorm[mini:maxi], ddof=ddofNorm, axis=None)
    # testing same fucntion gives p=100%
    #chi2Norm, pchi2Norm     = scipy.stats.chisquare(pdfnorm[mini:maxi], f_exp=pdfnorm[mini:maxi], ddof=ddofNorm, axis=None)
    txtChi2Norm             = "Chi-squared Test Normal :  DoF = {:1d}, chi² = {:5.1f}, p = {:2.1%}".format(dofNorm, chi2Norm, pchi2Norm)
    vprint(fncname + txtChi2Norm)

# END chi squared stuff  ------------------------------------------------------


# Kolmogorov-Smirnoff stuff ---------------------------------------------------
    #print("avgx: ", avgx)
    obs     = hist
    exp     = pdfs
    #print("blue values:\n", obs)
    #print("red  values:\n", exp)

    #print(scipy.stats.kstest(exp,'poisson', args=(20,), alternative='less'))

    ks_stats_p, ks_pval_p = scipy.stats.kstest(x, 'poisson', args=(avgx,))
    ks_stats_n, ks_pval_n = scipy.stats.kstest(x, 'norm'   , args=(avgx,))
    #print("========================= x_norm    : avg:", avgx, ks_stats_n, ks_pval_n)
    #print("========================= x_pois    : avg:", avgx, ks_stats_p, ks_pval_p)

    obs_cum  = np.empty_like(obs)
    exp_cum  = np.empty_like(exp)

    for i in range(0, len(obs) +1):
        obs_cum = np.cumsum(obs[0:i])
        exp_cum = np.cumsum(exp[0:i])

    #print("obs_cum: \n", obs_cum)
    #print("exp_cum: \n", exp_cum)

    #diff  = np.empty_like(obs)
    #for i in range(0, len(obs)):
    #    diff[i] = obs_cum[i] - exp_cum[i]
    #print("diff: \n", diff)
    #print("diff: \n", np.absolute(diff))

    #diffmax = np.max(np.absolute(diff))
    #print("diffmax: ", diffmax)

# END   Kolmogorov-Smirnoff stuff ---------------------------------------------------


    fig2 = plt.figure(facecolor = "#E7F9C9", dpi=gglobs.hidpiScaleMPL)
    vprint("plotPoisson: open figs count: {}, current fig: #{}".format(len(plt.get_fignums()), plt.gcf().number))

    plt.suptitle("Histogram with Poisson Fit", fontsize=12 )
    RsubTitle = DataSrc + "  Recs:" + str(x.size)
    plt.title(RsubTitle, fontsize=10, fontweight='normal', loc = 'right')
    plt.xlabel("Variable {}".format(yunit), fontsize=12)
    plt.ylabel("Frequency of Occurence", fontsize=12)
    plt.grid(True)
    plt.subplots_adjust(hspace=None, wspace=.2 , left=.17, top=0.85, bottom=0.15, right=.97)
    plt.ticklabel_format(useOffset=False)

    # canvas - this is the Canvas Widget that displays the `figure`
    # it takes the `figure` instance as a parameter to __init__
    canvas2 = FigureCanvas(fig2)
    canvas2.setFixedSize(600,400) # needs to be >=600 wide for the nav coords to show even when in 2 lines!


    # plot histogram and other curves #########################################

    # histogram
    plt.bar(bins[:-1],  hist,        color="cornflowerblue", align='center', width=step * 0.85,   label ="avg = {:0.2f}\nvar = {:0.2f}".format(avgx, varx))

    # Poisson curve
    plt.plot(bins[:-1], pdfs,        color='red',         linewidth=3,                        label ="r2  = {:0.3f}".format(r2))

    # Poisson curve residuals
    plt.plot(bins[:-1], hist - pdfs, color='orangered',   linewidth=1, marker='o', markersize=3, label="Residuals")

    if gglobs.stattest:
        # Normal curve
        plt.plot(bins[:-1], pdfnorm,        color='black', linewidth=1,                            label ="N r2 = {:0.3f}".format(r2N))

        # Normal curve residuals
        plt.plot(bins[:-1], hist - pdfnorm, color='0.3',   linewidth=1, marker='s', markersize=3,  label="N Residuals")

    # best place for Legend found with "best"!
    plt.legend(loc="best", fontsize=12, prop={"family":"monospace"})

    ###########################################################################

    labout  = QTextBrowser() # label to hold the description
    labout.setFont(gglobs.fontstd)
    labout.setLineWrapMode(QTextEdit.NoWrap)
    labout.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)
    labout.setMinimumHeight(330)
    #labout.setMinimumHeight(550)

    labout.append("Bin   Count Rate    Frequency    % of   Poisson-Fit    Residuals")

    if step == 1:
        # bins have only 1 count rate
        labout.append("No.                (blue col)   Total    (red line) (Freq - Fit)")
        for i in range(0, len(hist)):
            labout.append("{:3d}   {:4.1f}           {:8.1f}  {:5.2f}%    {:10.1f}   {:+10.1f}".format(i + 1, bins[i], hist[i], hist[i]*100 / lenx, pdfs[i], hist[i] - pdfs[i]))
    else:
        # bins have more than one count rate
        labout.append("No.   from ... to  (blue col)   Total    (red line) (Freq - Fit)")
        for i in range(0, len(hist)):
            labout.append("{:3d}   {:4.1f} ...{:4.1f}   {:8.1f}  {:5.2f}%    {:10.1f}   {:+10.1f}".format(i + 1, bins[i], bins[i+1] - 1, hist[i], hist[i]*100 / lenx, pdfs[i], hist[i] - pdfs[i]))

    labout.append("Total count=       {:10.1f}  100.00%   {:10.1f}   {:+10.1f}".format(sum(hist), sum(pdfs), sum(hist - pdfs)))

    labout.append("Countrates per Bin: {}".format(step))

    labout.append("\nGoodness of Fit Poisson :  r²  = {:5.3f}".format(r2))
    labout.append(txtChi2Poiss)

    if gglobs.stattest:
        labout.append("Goodness of Fit Normal  :  r²  = {:5.3f}".format(r2N))
        labout.append(txtChi2Norm)

#    labout.append("Kolmogorov-Smirnow Poisson Test: ")
#    labout.append("Poisson :  statistic = {:5.3f}, pvalue= {:2.3%}".format(ks_stats_p, ks_pval_p))
#    labout.append("Normal  :  statistic = {:5.3f}, pvalue= {:2.3%}".format(ks_stats_n, ks_pval_n))


# Assemble Data set statistics
    labout.append("\nData Set:")
    labout.append("File      = {}"    .format(DataSrc))
    labout.append("Records   = {}"    .format(x.size))
    labout.append("Cycletime ={:8.2f}".format(cycletime * 86400) + " sec (overall average)")
    labout.append("Average   ={:8.2f}".format(avgx))
    labout.append("Variance  ={:8.2f}  same as Average if true Poisson Dist.".format(varx))
    labout.append("Std.Dev.  ={:8.2f}".format(stdx))
    if avgx >= 0:
        labout.append("Sqrt(Avg) ={:8.2f}  same as Std.Dev. if true Poisson Dist.".format(np.sqrt(avgx)))
    else:
        labout.append("Sqrt(Avg) ={:8.2f}  same as Std.Dev. if true Poisson Dist.".format(gglobs.NAN))
    labout.append("Std.Err.  ={:8.2f}".format(stdx / np.sqrt(x.size)))
    labout.append("Skewness  ={:8.2f}  0:Norm.Dist.; skewed to: +:right   -:left".format(scipy.stats.skew    (x) ))
    labout.append("Kurtosis  ={:8.2f}  0:Norm.Dist.; shape is:  +:pointy: -:flat".format(scipy.stats.kurtosis(x) ))
    labout.append("")


    d       = QDialog()
    d.setWindowIcon(gglobs.iconGeigerLog)
    d.setWindowTitle("Poisson Test")
    #d.setWindowModality(Qt.ApplicationModal)
    #d.setWindowModality(Qt.NonModal)
    d.setWindowModality(Qt.WindowModal)

    #~mystatusBar = QStatusBar()
    #~mystatusBar.showMessage("")

    navtoolbar = NavigationToolbar(canvas2, d) # choice of parent does not matter?
    #~navtoolbar = NavigationToolbar(canvas2, None) # choice of parent does not matter?
    #~navtoolbar = NavigationToolbar(canvas2, gglobs.exgg) # choice of parent does not matter?

    # hide the cursor position from showing in the Nav toolbar
    #ax1 = plt.gca()
    #~ax1.format_coord = lambda x,y: f"x={x:.1f}, y={y:.1f}"
    #~ax1.format_coord = lambda x,y: "x={:.1f},\ny={:.1f}".format(x, y)
    #ax1.format_coord = lambda x,y: "x={:.1f}, y={:.1f}".format(x, y)

    bbox    = QDialogButtonBox()
    bbox.setStandardButtons(QDialogButtonBox.Ok)
    bbox.accepted.connect(lambda: d.done(0))

    #~layoutHtb = QHBoxLayout()
 #~#   layoutHtb.addWidget(bbox)
    #~layoutHtb.addWidget(navtoolbar)
    #~layoutHtb.addStretch()
    #~layoutHtb.addStretch() # strange - one is not enough? on Py3.5
    #~layoutHtb.addStretch() # strange - one is not enough? on Py3.8

    layoutV   = QVBoxLayout(d)
    layoutV.addWidget(navtoolbar)
    #layoutV.addLayout(layoutHtb)
    layoutV.addWidget(canvas2)
    #~layoutV.addWidget(mystatusBar)
    layoutV.addWidget(labout)
    layoutV.addWidget(bbox)

    setNormalCursor()

# show window
    fig2.canvas.draw_idle()
    setDebugIndent(0)
    d.exec()
    plt.close(fig2)




#! /usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
gglobs.py - GeigerLog's Super-Global Variables; can be read and modified
            in all scripts

include in programs with:
    import gglobs
"""

###############################################################################
#    This file is part of GeigerLog.
#
#    GeigerLog is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GeigerLog is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GeigerLog.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

__author__          = "ullix"
__copyright__       = "Copyright 2016, 2017, 2018, 2019, 2020, 2021"
__credits__         = [""]
__license__         = "GPL3"
__version__         = "1.2.2"             # version of next GeigerLog
__version__         = "1.2.2pre1"         #


# constants
NAN                 = float('nan')        # 'not-a-number'; used as 'missing value'
JULIANUNIX          = 2440587.5416666665  # julianday('1970-01-01 00:00:00') (UNIX time start)
JULIAN111           = 1721425.5 - 1       # julianday('0001-01-01 00:00:00') (matplotlib time start)
                                          # Why this minus 1? strange but needed

# counters
xprintcounter       = 0                   # the count of dprint, vprint, and wprint commands

# messages
python_failure      = ""                  # msg wrong Python version
startupProblems     = ""                  # msg configuration file missing or incorrect
configAlerts        = ""                  # msg ALERTs when reading config
notePadSearchText   = ""                  # last entered search text

# pointers
app                 = None                # points to app
exgg                = None                # points to ggeiger
notePad             = None                # points to print into notePad area
logPad              = None                # points to print into logPad area
btn                 = None                # points to the cycle time OK button; for inactivation
plotAudioPointer    = None                # points to the dialog window of plotaudio
plotScatterPointer  = None                # points to the dialog window of plotscatter
plotScatterFigNo    = None                # points to the window number of plotscatter
iconGeigerLog       = None                # points to the QIcon pixmap for 'icon_geigerlog.png'
bboxbtn             = None                # points to button box of GMC_popuo

# sql database
currentConn         = None                # connection to CURRENT database
logConn             = None                # connection to LOGGING database
hisConn             = None                # connection to HISTORY database

# flags
debug               = False               # helpful for debugging, use via command line
verbose             = False               # more detailed printing, use via command line
werbose             = False               # more verbose than verbose
devel               = False               # set some conditions for development CAREFUL !!!
devel1              = False               # =devel  + ADDITIONAL condition: load default db: default.logdb
devel2              = False               # =devel1 + ADDITIONAL condition: make connections
devel3              = False               # =devel2 + ADDITIONAL condition: quicklog
tput                = False               # to issue the tput command
redirect            = False               # on True redirects output from stdout and stderr to file stdlogPath
debugIndent         = ""                  # to indent printing
stopPrinting        = False               # to stop long prints of data

#special flags
testing             = False               # if true then some testing constructs will be activated CAREFUL !!!
graphdemo           = False               # if true then handling of line properties of selected var will be modified for demo
stattest            = False               # when True display statistics on Normal in Poisson test
test1               = False               # to run a specific test
test2               = False               # to run a specific test
test3               = False               # to run a specific test
test4               = False               # to run a specific test

# dir & file
dataDirectory       = "data"              # the data subdirectory to the program directory
gresDirectory       = "gres"              # where the icons and sounds reside
progName            = None                # program name geigerlog (or geigerlog.py)
progPath            = None                # path to program geigerlog file
dataPath            = None                # path to data dir
gresPath            = None                # path to icons and sounds dir
proglogPath         = None                # path to program log file geigerlog.proglog
stdlogPath          = None                # path to program log file geigerlog.stdlog
configPath          = None                # path to configuration file geigerlog.cfg
logFilePath         = None                # file path of the log file

binFilePath         = None                # file path of the bin file (GMC device)
datFilePath         = None                # file path of the bin file (Gamma Scout device)
AMFilePath          = None                # file path of the bin file (AmbioMon device, either *CAM or *.CPS file)
hisFilePath         = None                # file path of the his file

hisDBPath           = None                # file path of the his database file
logDBPath           = None                # file path of the log database file
currentDBPath       = None                # file path of the current DB file for log or his

fileDialogDir       = None                # the default for the FileDialog starts

HistoryDataList     = []                  # where the DB data are stored by makeHist
HistoryParseList    = []                  # where the parse comments are stored by makeHist
HistoryCommentList  = []                  # where the DB comments are stored by makeHist
activeDataSource    = None                # can be 'Log' or 'His'

manual_filename     = "auto"              # name of the included manual file, like "GeigerLog-Manual-v0.9.93pre17.pdf"
                                          # on 'auto' GL searches for a file beginning with 'GeigerLog-Manual'

# GUI options
window_width        = 1366                # the standard screen of 1366 x 768 (16:9),
window_height       = 768                 #
window_size         = "auto"              # 'auto' or 'maximized'
windowStyle         = "auto"              # may also be defined in config file
displayLastValsIsOn = False               # whether the displayLastValues windows is shown
hidpi               = False               # if true the AA_EnableHighDpiScaling will be applied
hipix               = False               # if true the AA_UseHighDpiPixmaps will be applied
hidpiActivation     = True                # The PyQt5 commands hidpi and hipix will be applied
hidpiScaleMPL       = 100                 # 100 is the default dpi for matplotlib


## Serial ports
    # pyserial Parity parameters
    # import serial :
    # serial.PARITY_NONE    = N
    # serial.PARITY_EVEN    = E
    # serial.PARITY_ODD     = O
    # serial.PARITY_MARK    = M
    # serial.PARITY_SPACE   = S
    #
    # finding pyserial supported parameters:
    # myser = serial.Serial()
    # myser           : Serial<id=0x7f7a05b18d68, open=False>(port=None, baudrate=9600, bytesize=8, parity='N',
    #                   stopbits=1, timeout=None, xonxoff=False, rtscts=False, dsrdtr=False)
    # myser.BAUDRATES : 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800, 9600,
    #                   19200, 38400, 57600, 115200, 230400, 460800, 500000, 576000, 921600,
    #                   1000000, 1152000, 1500000, 2000000, 2500000, 3000000, 3500000, 4000000
    # myser.BYTESIZES : 5, 6, 7, 8
    # myser.PARITIES  : 'N', 'E', 'O', 'M', 'S'
    # myser.STOPBITS  : 1, 1.5, 2

GMCbaudrates        = [1200, 2400, 4800, 9600, 14400, 19200, 28800, 38400, 57600,        115200]
I2Cbaudrates        = [            4800, 9600,        19200,        38400, 57600, 76800, 115200, 230400]
GSbaudrates         = [      2400,       9600,                                                           460800]

# Serial port for GMC devices
GMCser              = None                # serial port pointer
GMCbaudrate         = 115200              # will be overwritten by a setting in geigerlog.cfg file
GMCusbport          = '/dev/ttyUSB0'      # will be overwritten by a setting in geigerlog.cfg file
GMCtimeout          = 3                   # will be overwritten by a setting in geigerlog.cfg file
GMCtimeout_write    = 3                   # will be overwritten by a setting in geigerlog.cfg file
GMCttyS             = "ignore"            # to 'ignore' or 'include' '/dev/ttySN', N=1,2,3,... ports on USB Autodiscovery

# Serial port for I2C devices
I2Cser              = None                # serial port pointer
I2Cbaudrate         = 115200              # will be overwritten by a setting in geigerlog.cfg file
I2Cusbport          = '/dev/ttyUSB1'      # will be overwritten by a setting in geigerlog.cfg file
I2Ctimeout          = 3                   # will be overwritten by a setting in geigerlog.cfg file
I2Ctimeout_write    = 3                   # will be overwritten by a setting in geigerlog.cfg file
I2CttyS             = "ignore"            # to 'ignore' or 'include' '/dev/ttySN', N=1,2,3,... ports on USB Autodiscovery

# Serial port for Gamma-Scout devices
# NOTE: some responses of Gamma-Scout take almost 1 sec, timeout setting must be well above 1 sec limit!
GSser               = None                # serial port pointer
GSbaudrate          = 9600                # will be overwritten by a setting in geigerlog.cfg file
GSusbport           = '/dev/ttyUSB2'      # will be overwritten by a setting in geigerlog.cfg file
GStimeout           = 3                   # will be overwritten by a setting in geigerlog.cfg file
GStimeout_write     = 3                   # will be overwritten by a setting in geigerlog.cfg file
GSttyS              = "ignore"            # to 'ignore' or 'include' '/dev/ttySN', N=1,2,3,... ports on USB Autodiscovery
GSstopbits          = 1                   # generic in pyserial;     will not change in code
GSbytesize          = 7                   # specific to Gamma-Scout; will not change in code
GSparity            = "E"                 # specific to Gamma-Scout; will not change in code


## CALIBRATION

###############################################################################
## Beginning with version GeigerLog 1.0 the calibration is used as inverse of
## the previous definitions, i.e. 0.0065 µSv/h/CPM becomes 1/0.065 => 153.85 CPM/(µSv/h)
## This is rounded to 154 CPM/(µSv/h) to use no more than 3 valid digits.
###############################################################################

# The GMC factory default calibration factor is linear, i.e. the same
# for all 3 calibration points defined in a GMC counter.
# For a M4011 tube the calibration factor implemented in firmware is 0.0065 µSv/h/CPM
# For the 2nd tube in a GMC500+ device, different factors were seen:
# E.g.: from a calibration point #3 setting of 25CPM=4.85µSv/h the calibration
# factor is 0.194 µSv/h/CPM. While a calibration point #3 setting of
# 25CPM=9.75µSv/h results in 0.39 µSv/h/CPM!
# However, calibration determined in my own experiment reported here:
# http://www.gqelectronicsllc.com/forum/topic.asp?TOPIC_ID=5369
# see Reply #10, Quote: “With Thorium = 0.468, and K40 = 0.494,
# I'd finally put the calibration factor for the 2nd tube, the SI3BG,
# to 0.48 µSv/h/CPM. Which makes it 74 fold less sensitive than the M4011!”
# Others report even poorer values for the SI3BG:
# https://www.gqelectronicsllc.com/forum/topic.asp?TOPIC_ID=5554, Reply #17
# "The new correction factor for the primarily gamma tube is .881CPM/uSv/h or 1.135."
# This means calibration factor for that (2nd) tube = 1.135 µSv/h/CPM


# Default Calibration
# must be defined even if no connection exists. May be redefined in config.
DefaultSens1st     = 154                 # CPM/(µSv/h), =  0.0065 in units of µSv/h/CPM
DefaultSens2nd     = 2.08                # CPM/(µSv/h), =  0.48   in units of µSv/h/CPM
DefaultSens3rd     = 154                 # CPM/(µSv/h), =  0.0065 in units of µSv/h/CPM

# calibration of tubes #1, #2, #3
calibration1st      = DefaultSens1st     # calibration factor for the 1st tube
                                          # in most counters this is the only tube
                                          # in a GMC-500+ counter this is the 1st or high sensitivity tube
calibration2nd      = DefaultSens2nd     # calibration factor for the 2nd tube
                                          # in a GMC-500+ counter this is the 2nd or low-sensitivity tube
calibration3rd      = DefaultSens3rd     # calibration factor for the 3rd tube
                                          # to use more than 1 counter simultaneously; mapping set in config

# DEVICES

DevicesConnected    = 0                   # number of connected devices; determined upon connecting

# GMC                                     # will be set after reading the version of the counter from the device
GMCActivation       = False               # To use or not use a GMC Device counter
GMCConnection       = False               # Not connected if False
GMCDeviceName       = None                # to be detected in init; generic name like: "GMC Device"
GMCDeviceDetected   = None                # will be replaced after connection with full specific
                                          # name as detected, like 'GMC-300Re 4.20',
                                          # had been 14 chars fixed, can now be longer
GMC_variables       = "auto"              # which variables are natively supported
GMCLast60CPS        = None                # storing last 60 sec of CPS values
GMCEstFET           = None                # storing last 60 sec of CPS values for estimating the FET effect

# GMC Bugs and settings
GMC_memory          = "auto"              # Can be configured for 64kB or 1 MB in the config file
GMC_SPIRpage        = "auto"              # size of page to read the history from device
GMC_SPIRbugfix      = "auto"              # if True then reading SPIR gives one byte more than
                                          # requested  (True for GMC-300 series)
GMC_locationBug     = ["GMC-500+Re 1.18", # see the FIRMWARE BUGS topic
                       "GMC-500+Re 1.21"] # in the configuration file
GMC_FastEstTime     = "auto"              # if != 60 then false data will result
GMC_endianness      = "auto"              # big- vs- little-endian for calibration value storage
GMC_configsize      = "auto"              # 256 bytes in 300series, 512 bytes in 500/600 series
GMC_voltagebytes    = "auto"              # 1 byte in the 300 series, 5 bytes in the 500/600 series
GMC_nbytes          = "auto"              # the number of bytes the CPM and CPS calls, as well as
                                          # the calls to 1st and 2nd tube deliver ( 2 or 4)
GMCcfg              = None                # Configuration bytes of the counter. 256 bytes in 300series, 512 bytes in 500series
GMC_WifiIndex       = 0                   # the index column in cfgKeyHigh applicable for the current device
GMCsavedatatype     = "Unknown - will become known once the device is connected"
GMCsavedataindex    = None
GMC_WifiEnabled     = False               # does the counter have WiFi or not
GMC_SingleTube      = True                # all are single tube, except 500+, which is doublle tube


# AudioCounter
AudioActivation     = False               # Not available if False
AudioConnection     = False               # Not connected if False
AudioDeviceName     = None                # to be set in init
AudioDeviceDetected = None
AudioDevice         = None                # audiocard; to be set in init to (None, None) (could be USB, 9 and other)
AudioLatency        = "auto"              # to be set in init to (1.0, 1.0) sec latency
AudioPulseMax       = None                # +/- 32768 as maximum pulse height for 16bit signal
AudioPulseDir       = None                # False for negative; True for positive pulse
AudioThreshold      = None                # Percentage of pulse height to trigger a count
                                          # 60% of +/- 32768 => approx 20000
AudioRate           = None                # becomes 44100:  sampling rate audio; works for most sound cards
AudioChunk          = None                # becomes 32:     samples per read
AudioChannels       = None                # becomes (1,1):  1= Mono, 2=Stereo
AudioFormat         = None                # becomes (int16, int16): signed 16 bit resolution

AudioLastCpm        = 0                   # audio clicks CPM
AudioLastCps        = 0                   # audio clicks CPS
AudioLast60CPS      = None                # np array storing last 60 sec of CPS values
AudioThreadStop     = False               # True to stop the audio thread
AudioVariables      = "auto"              #
AudioSensitivity    = "auto"              # units: CPM / (µSv/h)
AudioMultiPulses    = None                # stores the concatenated audio data for plotting
AudioRecording      = None                # stores the Recording
AudioPlotData       = None                # stores the data to be plotted
AudioOei            = None                # the eia storage label


# RadMon stuff
RMActivation        = False               # must be true to use RadMon
RMConnection        = False               # True when initialized by GeigerLog
RMDeviceName        = None                # can currently be only RadMon+
RMDeviceDetected    = None                # can currently be only RadMon+
RMServerIP          = "auto"              # MQTT server IP, to which RadMon sends the data
RMServerPort        = "auto"              # Port of the MQTT server, defaults to 1883
RMServerFolder      = "auto"              # The MQTT folder as defined in the RadMon+ device
RMSensitivity       = "auto"              # calibration factor for the tube used
RMVariables         = "auto"              # a list of the variables to log, T,P,H,R
RMTimeout           = 3                   # waiting for "successful connection" confirmation
RMCycleTime         = 1                   # cycle time of the RadMon device
RMconnect           = (-99, "")           # flag to signal connection
RMdisconnect        = (-99, "")           # flag to signal dis-connection

rm_client           = None                # to be set in gdev_radmon.initRadMon
rm_cpm              = None                # temporary storage for CPM
rm_temp             = None                # temporary storage for Temperature
rm_press            = None                # temporary storage for Pressure
rm_hum              = None                # temporary storage for Humidity
rm_rssi             = None                # temporary storage for RSSI


# AmbioMon stuff
AmbioActivation     = False               # must be true to use AmbioMon++
AmbioConnection     = False               # True after being initialized by GeigerLog
AmbioDeviceName     = None                # can currently be only AmbioMon++
AmbioDeviceDetected = None                # can currently be only AmbioMon++
AmbioServerIP       = "auto"              # server Domain name or IP to which AmbioMon connects
#AmbioAP_IP          = "auto"              # IP address of Ambiomon in Access Point mode, Default = "192.168.4.1"
#AmbioSSID           = "auto"              # SSID of user's WiFi network
#AmbioSSIDPW         = "auto"              # Password of user's WiFi network
AmbioCalibration    = "auto"              # calibration factor for the tube used
AmbioVariables      = "auto"              # a list of the variables to log
AmbioDataType       = "auto"              # "LAST" or "AVG" for lastdata or lastavg
AmbioTimeout        = "auto"              # waiting for successful connection

# settings for the AmbioMon device
AmbioVoltage        = 444.44              # voltage of the GM tube in the AmbioMon
AmbioCycletime      = 1                   # cycle time of the AmbioMon device
AmbioFrequency      = 1500                # frequency of driving HV generator
AmbioPwm            = 0.5                 # Pulse Width Modulation of HV generator
AmbioSav            = "Yes"               # Switching into saving mode (Yes/No)


# Gamma-Scout stuff
GSActivation        = False               # Not available if False; to be set via config
GSConnection        = False               # Not connected if False
GSDeviceName        = None                # to be assigned in GS Init; now: "Gamma-Scout"
GSDeviceDetected    = None                # to be assigned in GS init with specific name, now same "Gamma-Scout"
GSVariables         = "auto"
GSSensitivity       = "auto"              # auto will become 108 in init for the LND712 tube
GStype              = None                # to be set in GS init as: Old, Classic, Online
GSFirmware          = None                # to be set in GS init
GSSerialNumber      = None                # to be set in GS init
GSDateTime          = None                # to be set when setting DateTime
GSusedMemory        = 0                   # to be set when calling History
GSMemoryTotal       = 65280               # max memory acc to manual, page 16, but when mem was 64450 (830 bytes free), interval was already 7d!
GSCalibData         = None                # the Gamma-Scout internal calibration data
GScurrentMode       = None                # "Normal", "PC", "Online"
GStesting           = False               # required to run GS under simulation; used only in module gdev_scout.py,


# I2C stuff
I2CActivation       = False               # Not available if False; to be set via config
I2CConnection       = False               # Not connected if False
I2CDeviceName       = None                # to be assigned in I2c Init; now:
I2CDeviceDetected   = None                # to be assigned in I2C init with specific name,
I2CVariables        = "auto"
I2CThread           = None                # Thread used for I2C
I2CSensor1          = None                # the BME280
I2CSensor2          = None                # the TSL259
I2CFirmware         = None                # to be set in init, but included in DeviceDetected
I2CSerialNumber     = None                # to be set in init, but not available
I2CInfo             = None                # to be set in init

# I2C Dongles
elv                 = None                # becomes instance of the ELV USB-I2C device

# I2C Sensors and Modules
#~bme280              = {
                       #~"name": "BME280",
                       #~"feat": "Temperature, Pressure, Humidity",
                       #~"addr": 0x77,      # (d119)  addr: 0x76, 0x77
                       #~"type": 0x60,      # (d96)   BME280 has chip_ID 0x60
                       #~"hndl": None,      # handle
                       #~"dngl": None,      # connected with dongle
                      #~}

#~tsl2591             = {
                       #~"name": "TSL2591",
                       #~"feat": "Light (Vis, IR)",
                       #~"addr": 0x29,      # (d41)  addr: 0x29
                       #~"type": 0x50,      # Device ID 0x50
                       #~"hndl": None,      # handle
                       #~"dngl": None,      # connected with dongle
                      #~}


# LabJack stuff
LJActivation        = False                # Not available if False
LJConnection        = False                # True when initialized by GeigerLog
LJDeviceName        = "None"               # LabJack U3 with probe EI1050
LJDeviceDetected    = "None"               # LabJack U3 with probe EI1050
LJVariables         = "auto"               # a list of the variables to log; auto=T,H,R
LJversionLJP        = "not in use"         # version of LabJackPython
LJversionU3         = "not in use"         # version of U3
LJversionEI1050     = "not in use"         # version of EI1050


# RaspiStuff
RaspiActivation     = False                # Not available if False
RaspiConnection     = False                # Not connected if False
RaspiDevice         = "None"               # to be set in init (the 'audio'device)
RaspiDeviceName     = "None"               # to be set in init
RaspiPulseMax       = 2**15                # +/- 32768 as maximum pulse height for 16bit signal
RaspiPulseDir       = False                # False for negative; True for positive pulse
RaspiThreshold      = 60                   # Percentage of pulse height to trigger a count
                                           # 60% of +/- 32768 => approx 20000
RaspiRate           = 44100                # sampling rate audio; works for most sound cards
RaspiChunk          = 32                   # samples per read
RaspiChannels       = 1                    # 1= Mono, 2=Stereo
RaspiFormat         = None                 # becomes (int16, int16)
RaspiLatency        = None                 # becomes setting in sec

RaspiLastCpm        = 0                    # audio clicks CPM
RaspiLastCps        = 0                    # audio clicks CPS
RaspiLast60CPS      = None                 # np array storing last 60 sec of CPS values
RaspiThreadStop     = False                # True to stop the audio thread
RaspiVariables      = "auto"               #
RaspiCalibration    = "auto"               # units: CPM / (µSv/h)
RaspiMultiPulses    = None                 # stores the concatenated audio data for plotting
RaspiRecording      = None                 # stores the Recording
RaspiPlotData       = None                 # stores the data to be plotted


# SimulCounter
SimulActivation     = False                # Not available if False
SimulConnection     = False                # Not connected if False
SimulDeviceName     = "None"               # to be set in init
SimulDeviceDetected = "None"               # to be set in init
SimulMean           = "auto"               # mean value of the Poisson distribution as CPSmean=10
SimulVariables      = "auto"               # default is CPM3rd, CPS3rd
SimulSensitivity    = "auto"               # units: CPM / (µSv/h)
SimulPredictive     = False                # not making a CPM prediction
SimulPredictLimit   = 25                   # CPM must reach at least this count before making prediction


# MiniMonCounter
MiniMonActivation     = False              # Not available if False
MiniMonConnection     = False              # Not connected if False
MiniMonDeviceName     = "None"             # to be set in init
MiniMonDeviceDetected = "None"             # to be set in init
MiniMonVariables      = "auto"             # default is T, X


#ESP32
terminal            = None                 # for the ESP32


# Logging Options
logging             = False                # flag for logging
logcycle            = 3                    # time in seconds between CPM or CPS calls in logging
lastValues          = None                 # last values received from device
lastRecord          = None                 # last records received from devices
LogReadings         = 0                    # counts readings since last logging start; prints as index in log
LogGetValDur        = NAN                  # duration to get all logged values
LogPlotDur          = NAN                  # duration to plot all logged values
LogTotalDur         = NAN                  # duration to get and plot all logged values

# History Options
keepFF              = False                # Flag in startup-options
                                           # Keeps all hexadecimal FF (Decimal 255) values as a
                                           # real value and not an 'Empty' one. See manual in chapter
                                           # on parsing strategy
fullhist            = False                # If False breaks history recording when 8k FF is read
                                           # if True reads the whole history from memory

# Graphic Options
allowGraphUpdate    = True                 # to block updates on reading data

Xleft               = None                 # time min if entered
Xright              = None                 # time max if entered
Xunit               = "Time"               # time unit; can be Time, auto, second, minute, hour, day
XunitCurrent        = Xunit                # current time unit

Ymin                = None                 # CPM min if entered
Ymax                = None                 # CPM max if entered
Yunit               = "CPM"                # CPM unit; can be CPM, CPS, µSv/h
YunitCurrent        = Yunit                # current count unit

Y2min               = None                 # Ambio min if entered
Y2max               = None                 # Ambio max if entered
Y2unit              = "Amb"                # Ambio unit °C or °F
Y2unitCurrent       = Yunit                # current Ambio unit

avgChecked          = False                # Plot the lines Average, +/- 95% confidence, and Legend with average +/- StdDev value
mavChecked          = False                # Plot the Moving Average line, and Legend with MovAvg length ins seconds and data points
mav_initial         = 60                   # length of Moving Average period in seconds
mav                 = mav_initial          # currently selected mav period
fprintMAV           = False                # to print to NotePad the moving average comments

# Plotstyle                                # placeholder; don't change here - is set in config
linestyle           = 'solid'              # overwritten by varsDefault
linecolor           = 'blue'               # overwritten by varsDefault
linewidth           = '1.0'                # configurable in geigerlog.cfg
markerstyle         = 'o'                  # configurable in geigerlog.cfg
markersize          = '15'                 # configurable in geigerlog.cfg

#Graph Y-Limits                            # to calculate cursor position for left, right Y-axis
y1_limits           = (0, 1)               # (bottom, top) of y left
y2_limits           = (0, 1)               # (bottom, top) of y right

# Data arrays and values
logTime             = None                 # array: Time data from total file
logTimeDiff         = None                 # array: Time data as time diff to first record in days
logTimeFirst        = None                 # value: Time of first record of total file

logTimeSlice        = None                 # array: selected slice out of logTime
logTimeDiffSlice    = None                 # array: selected slice out of logTimeDiff
logSliceMod         = None

sizePlotSlice       = None                 # value: size of plotTimeSlice

logDBData           = None                 # 2dim numpy array with the log data
hisDBData           = None                 # 2dim numpy array with the his data
currentDBData       = None                 # 2dim numpy array with the currently plotted data

# DEVICES properties

# holder for devices  of all supported devices with all attribs
Devices = {
           # Device-        Detected-    Var-        Active-
           # Name           Names        Names       Status
            "GMC"        : [None,        None,       False],    # 1
            "Audio"      : [None,        None,       False],    # 2
            "RadMon"     : [None,        None,       False],    # 3
            "AmbioMon"   : [None,        None,       False],    # 4
            "Gamma-Scout": [None,        None,       False],    # 5
            "I2C"        : [None,        None,       False],    # 6
            "LabJack"    : [None,        None,       False],    # 7
            "Raspi"      : [None,        None,       False],    # 8
            "MiniMon"    : [None,        None,       False],    # 9
            "Simul"      : [None,        None,       False],    # 10
          }

# VARIABLES - names and style

# colors at:  https://matplotlib.org/users/colors.html
# linestyles: "solid", "dotted"
varsDefault = { # do not modify this - use copy varsBook !!!!
#                                                        Units
#            (Short)     Long             Very short     CP* can be µSv/h     Style            Style
#            Names,      Names,           Names          T °C can be °F       Color            Line
#            key         0                1              2                    3                4
            "CPM"    : ["CPM",           "M",           "CPM",               "blue",          "solid" ],    # 1
            "CPS"    : ["CPS",           "S",           "CPS",               "magenta",       "solid" ],    # 2
            "CPM1st" : ["CPM1st",        "M1",          "CPM",               "deepskyblue",   "solid" ],    # 3
            "CPS1st" : ["CPS1st",        "S1",          "CPS",               "violet",        "solid" ],    # 4
            "CPM2nd" : ["CPM2nd",        "M2",          "CPM",               "cyan",          "solid" ],    # 5
            "CPS2nd" : ["CPS2nd",        "S2",          "CPS",               "#9b59b6",       "solid" ],    # 6
            "CPM3rd" : ["CPM3rd",        "M3",          "CPM",               "brown",         "solid" ],    # 7
            "CPS3rd" : ["CPS3rd",        "S3",          "CPS",               "orange",        "solid" ],    # 8
            "T"      : ["Temperature",   "T",           "°C",                "red",           "solid" ],    # 9
            "P"      : ["Pressure",      "P",           "hPa",               "black",         "solid" ],    # 10
            "H"      : ["Humidity",      "H",           "%",                 "green",         "solid" ],    # 11
            "X"      : ["Xtra",          "X",           "x",                 "#8AE234",       "solid" ],    # 12
          }

varsBook            = varsDefault.copy()   # need a copy to allow reset as colorpicker may change color

# Index,            DateTime,     CPM,     CPS,  CPM1st,  CPS1st,  CPM2nd,  CPS2nd,  CPM3rd,  CPS3rd,  Temp,   Press,   Humid,   X
# Column No         0             1        2     3        4        5        6        7        8        9       10       11       12
datacolsDefault     = 1 + len(varsDefault) # total of 13: DateTime plus 12 vars (index excluded; is not data),
#print("datacolsDefault: ",datacolsDefault)


# HOUSEKEEPING for VARIABLES

varcheckedCurrent   = {}                   # is variable checked for inclusion in plot
varcheckedLog       = {}                   # is variable checked for inclusion in plot for Log
varcheckedHis       = {}                   # is variable checked for inclusion in plot for History
varlabels           = {}                   # info on avg, StdDev etc; will be set in gsup_plot
loggableVars        = {}                   # Vars which can be logged with current connections

for vname in varsBook.keys():
    varcheckedLog       [vname] = False    # set all false
    varcheckedHis       [vname] = False    # set all false
    varcheckedCurrent   [vname] = False    # set all false
    varlabels           [vname] = ""       # set all to empty
    loggableVars        [vname] = False    # set all False

activeVariables     = 0                    # determined in switchConnections; at least 1 needed for logging
textDevVars         = ""                   # determined in switchConnections, Devices with associated variables
varMap              = {}                   # holds mapping of the variables


# SCALING

# Value - Scaling - this modifies the measured variable;
# the modification will be stored to Database
ValueScale           = {}
ValueScale["CPM"]    = "VAL"               # no scaling
ValueScale["CPS"]    = "VAL"               # no scaling
ValueScale["CPM1st"] = "VAL"               # no scaling
ValueScale["CPS1st"] = "VAL"               # no scaling
ValueScale["CPM2nd"] = "VAL"               # no scaling
ValueScale["CPS2nd"] = "VAL"               # no scaling
ValueScale["CPM3rd"] = "VAL"               # no scaling
ValueScale["CPS3rd"] = "VAL"               # no scaling
ValueScale["T"]      = "VAL"               # no scaling
ValueScale["P"]      = "VAL"               # no scaling
ValueScale["H"]      = "VAL"               # no scaling
ValueScale["X"]      = "VAL"               # no scaling

# Graph Sscaling - this does ONLY modify how the data it is plotted
# the modification will NOT be stored to Database
GraphScale           = {}
GraphScale["CPM"]    = "VAL"               # no scaling
GraphScale["CPM1st"] = "VAL"               # no scaling
GraphScale["CPM2nd"] = "VAL"               # no scaling
GraphScale["CPS"]    = "VAL"               # no scaling
GraphScale["CPS1st"] = "VAL"               # no scaling
GraphScale["CPS2nd"] = "VAL"               # no scaling
GraphScale["CPM3rd"] = "VAL"               # no scaling
GraphScale["CPS3rd"] = "VAL"               # no scaling
GraphScale["T"]      = "VAL"               # no scaling
GraphScale["P"]      = "VAL"               # no scaling
GraphScale["H"]      = "VAL"               # no scaling
GraphScale["X"]      = "VAL"               # no scaling


# HELP DETAILS

helpOptions = """
Usage:  geigerlog [Options] [Commands]

Options:
    -h, --help          Show this help and exit.
    -d, --debug         Run with printing debug info.
                        Default is debug = False
    -v, --verbose       Be more verbose.
                        Default is verbose = False.
    -w, --werbose       Be much more verbose.
                        Default is werbose = False.
    -V, --Version       Show version status and exit.
    -P, --Portlist      Show available USB-to-Serial ports
                        and exit.
    -R  --Redirect      Redirect stdout and stderr to
                        file geigerlog.stdlog (for debugging).
    -s  --style name    Sets the style; see also manual and
                        Command: 'showstyles'.
                        Default is set by your system

Commands:
    showstyles          Show a list of styles available on
                        your system and exit. For usage details see manual.
    keepFF              GMC counter only: Keeps all hexadecimalFF (Decimal 255)
                        values as a real value and not an 'Empty' one. See
                        manual in chapter on parsing strategy.

By default, data files will be read-from/written-to the data directory
"data", which is a subdirectory to the program directory

To watch debug and verbose output start the program from the
command line in a terminal. The output will print to the terminal.

With the Redirect option all output - including Python error
messages - will go into the redirect file geigerlog.stdlog.
"""

helpQuickstart = """
<!doctype html>
<style>
    h3          { color: darkblue; margin:10px 0px 5px 0px; padding:0px; }
    p, ol, ul   { margin:8px 0px 4px 0px; padding:0px;}
</style>
<H3>Mouse Help</H3><p>resting your mouse cursor briefly over an item of the window shows a ToolTip message providing info on the item!

<h3>USB Port and Baudrate</h3>
<p>Assuming a USB cable is connecting computer and device:</p>
<p>Start GeigerLog and from the menu choose <b>Help -> Show & Select USB Port and
Baudrate</b>, or press the blue <b>Help Port</b> toolbar button.
Select the proper settings for your device, and press ok.

<p>If you don't know the proper settings you may start the <b>USB Autodiscovery</b>
from the Help menu for your device, but it is strongly suggested that you first
read the manual in the chapters for your device and heed the CAUTION message popping up!</p>

<h3>Start Logging</h3>
<p>This will create a database file to store newly acquired data from the devices.</p>
<ol>
<li>Connect the computer with the devices<br>(menu: <b>Device -> Connect Devices</b>, or click the <b>Connect</b> button)</li>
<li>GMC device only: Switch device power on<br>(menu: <b>Device -> GMC Series -> Switch Power ON</b>))</li>
<li>Load a log file<br>(menu: <b>Log -> Get Log File or Create New One</b>)<br>
In the Get Log File dialog enter a new filename and press Enter,
or select an existing file to <b>APPEND</b> new data to this file.</li>
<li>Start logging<br>(menu: <b>Log -> Start Logging</b>, or click the <b>Start Log</b> toolbar icon)</li>
</ol>

<h3>Get History</h3>
<p>This will create a database file to store all data downloaded from the internal storage of a device.</p>
<ol>
<li>Connect the computer with the devices<br>(menu: <b>Device -> Connect Devices</b>, or click the <b>Connect</b> button)</li>
<li>Get history from device<br>(menu: <b>History -> &lt;Your device&gt; -> Get History from Device</b>)</li>
<li>Enter a new filename (e.g. enter 'myfile' and press Enter) or select an existing file
<br>Note: selecting an existing file will <b>OVERWRITE</b> this file!</li>
</ol>


<h3>Graphical Analysis</h3>
<p>To zoom into the graph either enter the Time Min and
Time Max values manually, or, much easier, do a mouse-left-click to the left
of the feature in the graph and right-click to the right, and then click the Apply button.</p>

<p>Use other features as detailed in the manual.</p>

<h3>GeigerLog Manual</h3>
<p>Available locally (menu: <b>Help -> GeigerLog Manual</b>) and <a href="https://sourceforge.net/projects/geigerlog/files/">online</a>
at SourceForge project GeigerLog: <a href="https://sourceforge.net/projects/geigerlog/">https://sourceforge.net/projects/geigerlog/</a></p>
"""


helpFirmwareBugs = """
<!doctype html>
<style>
    h3              { color: darkblue; margin:5px 0px 5px 0px; padding:0px; }
    p, ol, ul, li   { margin:4px 0px 10px 0px; padding:0px;}
    td, th          { padding: 0px 30px 0px 0px;}
</style>

<h3>Firmware bugs in some devices</h3>

<p><b>Device: GMC-500+</b></p>

<p>When this device is on firmware 1.18 it identifies itself with an empty identifier,
which prohibits use on GeigerLog. It should identify itself as 'GMC-500+Re 1.18'.</p>

<p>A work-around is to cycle the connection: ON -> OFF -> ON. Then it should work.</p>

<p>This bug is corrected with firmware 1.21. It is recommended to contact GMC support for an update.
The Geiger counter will then identify itself as 'GMC-500+Re 1.21'.</p>
<p>-------------------------------------------------------------------------------</p>
<p><b>Device: GMC-500 / 600 series</b></p>

<p>Some firmware versions, at least 1.18 and 1.21, have a bug (called 'location bug') which
results in errors in the parsing of the history file. It is advised to upgrade at least to
the 1.22 firmware!</p><p>-------------------------------------------------------------------------------</p>

<p><b>"Fast EstimateTime"</b></p>

<p>This mode is active by default in its worst setting! It invokes an algorithm
 - not disclosed by GQ - which results in a severe distortion of all CPM
 recordings, and may even create counts when there are none.</p>

<p>Present as a configuration setting in Device GMC-500+ version 2.24 and
likely in all other models with a firmware supporting this mode.</p>

<p>It has settings of 60, 30, 20, 15, 10 and 5 seconds, plus a 'dynamic' setting,
which apparently simply stands for 3 seconds. The latter is the default and the
worst setting.</p>

<p>It looks like this algorithm is switched off at 60 seconds, therefore that
setting is strongly recommended! GeigerLog gives a warning when a GMC counter is
activated with this setting at anything besides 60 seconds.</p>

<p>A firmare correction has NOT been announced.</p>

"""


helpWorldMaps = """
<!doctype html>
<style>
    h3          { color: darkblue; margin:5px 0px 5px 0px; padding:0px; }
    p, ol, ul   { margin:4px 0px 4px 0px; padding:0px;}
</style>

<H3>Radiation World Maps</H3>

<p>Several web sites exist, which attempt to show a worldwide map of the
<b>BACKGROUND</b> radioactivity, hoping to be of help to the people in case of
a nuclear emergency, which will result in elevated levels of radioactivity.
Some are run by governments, others by enthusiastic hobbyists.</p>

<p>Among the latter ones are:</p>

<ul>
<li><a href='http://www.gmcmap.com/' >gmcmap.com</a> - This is the one supported by GQ Electronics</li>
<li><a href='https://radmon.org/index.php' >radmon.org</a> - Opened again after hacking episode</li>
<li><a href='https://blog.safecast.org/' >safecast.org</a> - Accepting radiation as well as air quality data</li>
</ul>

<p>Currently only GQ's GMCmap is supported by GeigerLog; others may follow.</p>

<p>GQ suggests to use your Geiger counter (versions with WiFi, i.e.
GMC-320+V5, GMC-500, GMC-600 series) to directly update their website.
See the GeigerLog manual for why this is actually not such a good idea.</p>

<p>But you can also support their worldmap using GeigerLog, and not only
provide more meanigful data, but use any of their non-WiFi counters just as
well.</p>

<p>If you want to contibute to
<a href='http://www.gmcmap.com/' >gmcmap.com</a>, you need to register there.
This provides you with a UserID and a CounterID. Enter both into the
respective fields in the GeigerLog configuration file 'geigerlog.cfg' under
the heading 'Worldmaps'. That's it!</p>

<p>You need to be logging. Then the toolbar icon 'Map' turns blue, aka it
becomes enabled (as well as the menu entry Web -> Update Radiation World Map).
Click the icon and you'll be presented with a dialogue box, allowing you to
cancel or to send your data to the map. Obviously, for this to
succeed you need to have an active internet connection at your computer!</p>

<p>You will see a confirmation printed to GeigerLog's NotePad area, including
the response of the website.</p>

<H3>A word of caution:</H3>
<p>As far as I can see there is no quality control of the data! Nothing
prevents users from putting a strong radioactive source in front of their detector,
and pushing these data to the web. In fact, you don't even need a counter, and
don't even need GeigerLog, but can enter any data you wish manually!
Also, there does not seem to be a way to retract any inadvertently
sent data. Poor data will quickly destroy any value of those sites.</p>

<p>Even more subtle things may reduce the value of those websites: Geiger
counter readings fluktuate quite significantly. Thus when individual, single
readings are posted, the values maybe significantly higher or lower than the
average, suggesting changes that don't exist.</p>

<p>GeigerLog will always send the average of ALL data you see in the plot in
the moment you press the Map button. Thus you can easily select or cut out
data which have nothing to do with the background. And
GeigerLog uses this average for both CPM and ACPM, and calculates uSV based
on it. I suggest to have values assembled for at least 30 minutes,
more is better, before sending anything to the maps. Governmental sites like this
<a href='https://www.naz.ch/de/aktuell/tagesmittelwerte.shtml'>Swiss site</a>
provide even only DAILY averages of quality controlled data! See the GeigerLog
manual for a broader discussion.
"""


helpOccupationalRadiation = """
<!doctype html>
<style>
    h3              { color: darkblue; margin:5px 0px 5px 0px; padding:0px; }
    p, ol, ul, li   { margin:4px 0px 10px 0px; padding:0px;}
    td, th          { padding: 0px 30px 0px 0px;}
</style>

<H3>Occupational Radiation Limits</H3>

<p>The exposure to radiation is strongly regulated all over the world. As examples,
the occupational limits are given for USA and Germany.</p>

<p>"Occupational" refers to people working in fields with typically higher exposure to radiation
compared to the average person, like medical people applying X-rays, workers in nuclear power plants,
people in aviation, people in mining.</p>

<p>Of the many limits specified, only the yearly and lifelong exposures are given
here; the links will guide you to sites with more extensive specifications.<br></p>

<table >
  <tr>
    <th >    </th>
    <td ><b>Germany</b></td>
    <td ><b>USA</b></td>
  </tr>
  <tr>
    <td >Yearly exposure</td>
    <td >  20 mSv</td>
    <td >  50 mSv</td>
  </tr>
  <tr>
    <td >(equiv. to permanent:</td>
    <td >  2.3 µSv/h</td>
    <td >  5.7 µSv/h)</td>
  </tr>
  <tr>
    <td >Lifelong exposure</td>
    <td > 400 mSv</td>
    <td >2350 mSv</td>
  </tr>
  <tr>
    <td >Links</td>
    <td><a href='https://www.bfs.de/DE/themen/ion/strahlenschutz/beruf/grenzwerte/grenzwerte.html'>BfS Grenzwerte</a></td>
    <td><a href='https://www.osha.gov/SLTC/radiationionizing/introtoionizing/ionizingattachmentsix.html'>OSHA</a></td>
  </tr>
</table>

<p>The differences are quite significant; see details in the links.</p>
"""


helpAbout = """
<!doctype html>
<style>
    h3              { color: darkblue; margin:5px 0px 5px 0px; padding:0px; }
    p, ol, ul, li   { margin:4px 0px 10px 0px; padding:0px;}
    td, th          { padding: 0px 30px 0px 0px;}
</style>

<p><span style='color:darkblue; font:bold; font-size:x-large;'>GeigerLog</span>
is a combination of data logger, data presenter, and data analyzer.</p>

<p>It is based on Python (Version 3), hence it runs on Linux, Windows, Macs,
and other systems.</p>

<p>GeigerLog had initially been developed for the sole use with Geiger counters,
but has now become a more universal tool, which equally well handles
environmental data like temperature, air-pressure, and humidity, and is prepared
for future sensors.</p>

<p>In its present state it can e.g. be deployed as a monitor for a remote
weather station, complemented with a Geiger counter to monitor radioactivity,
and an In-house CO2 monitoring system.</p>

<p><br>Currently Supported Devices:</p>

<p><b>GMC Devices:</b><br>GeigerLog continuous to support GQ Electronics's
GMC-3xx, GMC-5xx, and GMC-6xx line of classical Geiger counters, including the
variants with an additional 2nd Geiger tube.</p>

<p><b>AudioCounter Devices:</b>
<br>GeigerLog supports Geiger counters which only give an audio click for a
radioactive event.</p>

<p><b>RadMon Devices:</b>
<br>GeigerLog supports the RadMon+ hardware, which can provide a Geiger counter
as well as an environmental sensor for temperature, air-pressure
(atmospheric-pressure), and humidity.</p>

<p><b>AmbioMon Devices:</b>
<br>GeigerLog supports AmbioMon++ hardware, devices which can provide a Geiger
counter as well as an environmental sensor for temperature, air-pressure
(atmospheric-pressure), and humidity.</p>

<p><b>Gamma-Scout Devices:</b>
<br>GeigerLog supports the Geiger counters from Gamma-Scout.</p>

<p><b>LabJack Devices:</b>
<br>GeigerLog supports the u3 and Ei1050 hardware from LabJack.</p>

<p><b>I2C Devices:</b>
<br>GeigerLog supports devices connected by the I2C interface</p>

<p><b>MiniMon Devices - CO2 Monitoring:</b>
<br>GeigerLog supports these In-house CO2 monitoring devices (Linux only)</p>

<p><b>SimulCounter Devices:</b>
<br>A simulated Geiger counter getting its “counts” from a Poisson random
number generator.</p>

<p><br>The most recent version of GeigerLog as well as an up-to-date
GeigerLog-Manual can be found at project GeigerLog at Sourceforge:
<a href="https://sourceforge.net/projects/geigerlog/">
https://sourceforge.net/projects/geigerlog/</a>.</p>

<p>References:</p>
<table>
<tr><td>&nbsp;&nbsp;Author:                         </td> <td>%s</td></tr>
<tr><td>&nbsp;&nbsp;Version GeigerLog:&nbsp;&nbsp;  </td> <td>%s</td></tr>
<tr><td>&nbsp;&nbsp;Copyright:                      </td> <td>%s</td></tr>
<tr><td>&nbsp;&nbsp;License:                        </td> <td>%s see included
file 'COPYING' and <a href="http://www.gnu.org/licenses">http://www.gnu.org/licenses</a></td></tr>
</table>
<br>
"""
